function yasearch(core) {
    this._barCore = core;
    this._logger = this._barCore.Log4Moz.repository.getLogger(core.appName + ".App");
    this._overlayControllers = {};
    this._init();
}

yasearch.prototype = {
    get widgetsRegistry() {
        return this._widgetsRegistry;
    },
    
    get defaultWidgetsInfo() {
        return this._presetWidgetsInfo;
    },
    
    get name() {
        return this._barCore.appName;
    },
    
    getNewControllerID: function barApp_getNewControllerID(controller) {
        var newCID = this._newCID++;
        this._overlayControllers[newCID] = controller;
        return newCID;
    },
    
    forgetController: function barApp_forgetController(CID) {
        delete this._overlayControllers[CID];
    },
    
    findOverlayController: function barApp_findOverlayController(CID) {
        return this._overlayControllers[CID];
    },
    
    getNewWidgetInstanceID: function barApp_getNewWidgetInstanceID() {
        return "" + (new Date()).getTime() + this._newWID++;
    },
    
    get core() {
        return this._barCore;
    },
    
    get localeString() {
        if (!this._localeString) {
            try {
                let xulChromeReg = Cc["@mozilla.org/chrome/chrome-registry;1"].getService(Ci.nsIXULChromeRegistry);
                this._localeString = xulChromeReg.getSelectedLocale(APP_NAME);
            } catch(e) {
                return "ru";
            }
        }
        
        return this._localeString;
    },
    
    navigate: function barApp_navigate(wnd, url, target, yandexAction) {
        var statData = undefined;
        if (yandexAction)
            statData = {action: yandexAction};
        wnd.Ya.loadURI(url, target, statData);
    },
    
    finalize: function barApp_finalize() {
        this._overlayProvider.clear();
        this._overlayProvider = null;
        
        this._basePackage.finalize();
        this._basePackage = null;
        this._widgetLibrary = null;
        
        this._barCore.Lib.XB.finalize();
        
        this._overlayControllers = null;
        this._widgetsRegistry = null;
        this._presetWidgetsInfo = null;
        
        this._logger = null;
        this._barCore = null;
    },
    
    _init: function barApp_init() {
        this.defaultIDsHash = {};
        
        this._barCore.Lib.XB.init(this);
        
        this._widgetsRegistry = new this._barCore.Lib.XB.WidgetsRegistry();        
        
        var cbContentDirFile = this._barCore.extensionPathFile;
        cbContentDirFile.append("chrome");
        cbContentDirFile.append("content");
        cbContentDirFile.append("custombar");
        
        var baseLibFile = cbContentDirFile.clone();
        baseLibFile.append("basepkg");
        this._basePackage = new this._barCore.Lib.XB.WidgetPackage(baseLibFile);
        this._widgetLibrary = this._basePackage.getLibrary("baselib");
        
        this._widgetsRegistry.registerLibrary(this._widgetLibrary);
        
        /*
        var widgetPreset = new this._barCore.Lib.XB.Preset();
        var presetFile = cbContentDirFile.clone();
        presetFile.append("base_preset.xml");
        try {
            widgetPreset.loadFromFile(presetFile);
            this._presetWidgetsInfo = widgetPreset.widgetsInfo;
        }
        catch (e) {
            this._logger.error("Failed loading default preset.\n" + this._barCore.formatExceptionMessage(e));
            this._logger.debug("Stack trace:\n" + e.stack);
        }
        */
        var isUkranianLocale = /^uk(\-UA)?$/.test(this.localeString);
        var defQID1 = isUkranianLocale? 3005: 1;
        var defQID2 = isUkranianLocale? 3027: 23;
        this._presetWidgetsInfo = [
            {
                protoID: "A78155FC-41D5-4304-BD60-A01D8C1FB9FA",
                settings: {"quote-id": defQID1}
            },
            {
                protoID: "A78155FC-41D5-4304-BD60-A01D8C1FB9FA",
                settings: {"quote-id": defQID2}
            }
        ];
        
        this._overlayProvider = new this._barCore.Lib.OverlayProvider(this);
        var appProtocol = Cc["@mozilla.org/network/protocol;1?name=" + this._barCore.appName]
            .getService(Ci.nsIProtocolHandler).wrappedJSObject;
        appProtocol.addDataProvider(this._overlayProvider);
    },
    
    _barCore: null,
    _logger: null,
    _basePackage: null,
    _widgetsRegistry: null,
    _widgetLibrary: null,
    _presetWidgetsInfo: null,
    _overlayProvider: null,
    _newCID: 0,
    _newWID: 0,
    _overlayControllers: null,
    _localeString: null
};
