function OverlayProvider(barApplication) {
    this._application = barApplication;
    this._ItemsIdPrefix = barApplication.name + ".cb-";
    this._unicodeConverter = Components.classes["@mozilla.org/intl/scriptableunicodeconverter"]
                                       .createInstance(Components.interfaces.nsIScriptableUnicodeConverter);
    this._unicodeConverter.charset = "UTF-8";
    this._DOMSerializer = Components.classes["@mozilla.org/xmlextras/xmlserializer;1"]
                                    .getService(Components.interfaces.nsIDOMSerializer);
    
    this._overlayDocScel = ' \
        <!DOCTYPE overlay [ \
            <!ENTITY app.name "' + this._application.name + '"> \
            <!ENTITY % barDTD SYSTEM "chrome://' + this._application.name + '/locale/custombar.dtd"> \
            %barDTD; \
        ]> \
        <overlay xmlns="http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul"> \
          <script type="application/x-javascript" src="chrome://&app.name;/content/custombar/overlay.js"/> \
          <toolbarpalette id="BrowserToolbarPalette"/> \
        </overlay>';
    
    this._logger = barApplication.core.Log4Moz.repository.getLogger(barApplication.name + ".DynOverlay");
};

OverlayProvider.prototype = {
    constructor: OverlayProvider,
    
    clear: function Overlay_clear() {
        this._overlayDoc = null;
    },
    
    newURI: function Overlay_newURI(aSpec, aOriginalCharset, aBaseURI) {
        if (!this._canHandleSpec(aSpec))
            return null;
        
        let content = "";
        try {
            content = btoa(this._getBrowserOverlay());
        }
        catch (e) {
            this._logger.error(e);
            this._logger.debug(e.stack);
        }
        
        let uri =  Cc["@mozilla.org/network/simple-uri;1"].createInstance(Ci.nsIURI);
        uri.spec = "data:application/vnd.mozilla.xul+xml;base64," + content;
        
        return uri;
    },
    
    _overlayDocScel: undefined,
    _ItemsIdPrefix: undefined,
    _unicodeConverter: null,
    _DOMSerializer: null,
    _overlayDoc: null,
    _logger: null,
    
    _canHandleSpec: function Overlay__canHandleSpec(aSpec) {
        return (aSpec && aSpec.toLowerCase() === "browser-overlay");
    },
    
    _getBrowserOverlay: function Overlay__getBrowserOverlay() {
        //if (!this._overlayDoc)
            this._overlayDoc = this._createBrowserOverlay();
        return this._unicodeConverter.ConvertFromUnicode(this._DOMSerializer.serializeToString(this._overlayDoc));
    },
    
    _createBrowserOverlay: function Overlay__createBrowserOverlay() {
        const DOMParser = Cc["@mozilla.org/xmlextras/domparser;1"].getService(Ci.nsIDOMParser);
        
        var overlayDoc = DOMParser.parseFromString(this._overlayDocScel, "text/xml");
        var currentSetIDsData = this._currentSetIds;
        var defaultSetStr = Components.classes["@yandex.ru/yasearch;1"]
            .getService(Components.interfaces.nsIYaSearch).wrappedJSObject.yaOverlay.defaultset;
        var defaultSetIDs = defaultSetStr.split(",");
        var presetIDPattern = new RegExp(this._application.name + "\\.cb\\-default\\-(\\d+)$");
        var presetIDs = [];
        for each (let defaultSetID in defaultSetIDs) {
            if (defaultSetID.match(presetIDPattern)) {
                presetIDs.push(defaultSetID);
            }
        }
        
        var toolbarPalette = overlayDoc.getElementById("BrowserToolbarPalette");
        for each (let protoID in this._application.widgetsRegistry.registeredProtoIDs) {
            var widgetProto = this._application.widgetsRegistry.findProto(protoID);
            var widgetIDPrefix = this._ItemsIdPrefix + widgetProto.getID() + "-inst-";
            var iconPath = widgetProto.getIconPath();
            var widgetIconUrl = iconPath? widgetProto.getLibrary().libPackage.getXBURL(iconPath): "";
            
            var protoInstArray = currentSetIDsData[widgetProto.getID()];
            if (protoInstArray) {
                for each (let instID in protoInstArray) {
                    var toolbaritemId = widgetIDPrefix + instID;
                    toolbarPalette.appendChild(this._makeToolbarItem(overlayDoc,
                                                                     toolbaritemId,
                                                                     widgetProto.getName(),
                                                                     widgetIconUrl) );
                    
                    this._application.getNewWidgetInstanceID();
                    
                    if (widgetProto.isUnique())
                        break;
                }
            }
            else {
                toolbarPalette.appendChild(this._makeToolbarItem(overlayDoc,
                                                                 widgetIDPrefix + "0",
                                                                 widgetProto.getName(),
                                                                 widgetIconUrl));
            }
        }
        
        var presetItemsInfo = XB._base.application.defaultWidgetsInfo;
        for each (let presetID in presetIDs) {
            var presetIDMatch = presetID.match(presetIDPattern);
            
            var presetItemIndex = presetIDMatch[1];
            var presetItemInfo = presetItemsInfo[presetItemIndex];
            if (!presetItemInfo) continue;
            var widgetProto = this._application.widgetsRegistry.findProto(presetItemInfo.protoID);
            var iconPath = widgetProto.getIconPath();
            var widgetIconUrl = iconPath? widgetProto.getLibrary().libPackage.getXBURL(iconPath): "";
            toolbarPalette.appendChild(this._makeToolbarItem(overlayDoc,
                                                             presetID,
                                                             widgetProto.getName(),
                                                             widgetIconUrl));
        }
        
        this._cleanupPreferences();
        
        return overlayDoc;
    },
    
    _makeToolbarItem: function Overlay_makeToolbarItem(overlayDoc, aId, aName, aImage) {
        const XULNS = "http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul";
        let toolbarItem = overlayDoc.createElementNS(XULNS, "toolbaritem");
        toolbarItem.setAttribute("id", aId);
        toolbarItem.setAttribute("label", aName);
        toolbarItem.setAttribute("image", aImage);
        
        return toolbarItem;
    },
    
    get _currentSetIds() {
        let itemsIdPrefixRE = sysutils.RegExp.escape(this._ItemsIdPrefix);
        const currSetIDRE = new RegExp(itemsIdPrefixRE + "(.+)\\-inst\\-(.+)");
        
        var result = {};
        
        var rdfService = Components.classes["@mozilla.org/rdf/rdf-service;1"]
                                   .getService(Components.interfaces.nsIRDFService);
        var localStoreDataSource = rdfService.GetDataSource("rdf:local-store");
        
        var allResources = localStoreDataSource.GetAllResources();
        var currentSetResource = rdfService.GetResource("currentset");
        while (allResources.hasMoreElements()) {
            let res = allResources.getNext().QueryInterface(Ci.nsIRDFResource);
            let tool = res.Value;
            
            if (tool) {
                var toolbarResource = rdfService.GetResource(tool);
                var currentSetTarget = localStoreDataSource.GetTarget(toolbarResource, currentSetResource, true);
                var currentSetStr = "";
                if (currentSetTarget instanceof Components.interfaces.nsIRDFLiteral)
                    currentSetStr = currentSetTarget.Value;
                
                if (currentSetStr) {
                    var currentSetIds = currentSetStr.split(",");
                    for (let i = 0, len = currentSetIds.length; i < len ; i++) {
                        var barWidgetIDMatch = currentSetIds[i].match(currSetIDRE);
                        if (barWidgetIDMatch) {
                            var widgetProtoID = barWidgetIDMatch[1];
                            var widgetInstance = barWidgetIDMatch[2];
                            var instArray = result[widgetProtoID];
                            if (!instArray) {
                                instArray = [];
                                result[widgetProtoID] = instArray;
                            }
                            instArray.push(widgetInstance);
                        }
                    }
                }
            }
        }
        
        return result;
    },
    
    _getToolbarForNode: function(aNode) {
        let toolbar = false,
            node = aNode;
        
        while (node && !toolbar) {
            node = node.parentNode;
            if (node && node.localName == "toolbar")
                toolbar = node;
        }
        
        return toolbar;
    },
    
    _persistToolbarSet: function Overlay__persistToolbarSet(aToolbar) {
        if (aToolbar && aToolbar.localName && aToolbar.localName == "toolbar") {
            aToolbar.setAttribute("currentset", aToolbar.currentSet);
            aToolbar.ownerDocument.persist(aToolbar.id, "currentset");
        }
    },
    
    _getAllBrowsers: function Overlay__getBrowsers() {
        let windowMediator = Cc["@mozilla.org/appshell/window-mediator;1"].getService(Ci.nsIWindowMediator),
            browsers = [],
            enumerator = windowMediator.getEnumerator("navigator:browser");
        
        while (enumerator.hasMoreElements())
            browsers.push(enumerator.getNext());
        
        return browsers;
    },
    
    removeWidget: function Overlay_removeToolbarItem(aWidget, aPersist, aBrowser) {
        let toolbaritemId;
        
        if (aWidget instanceof Ci.nsIDOMElement)
            toolbaritemId = aWidget.id;
        else if (aWidget instanceof XB.WidgetLibrary.WidgetInstance)
            toolbaritemId = this._ItemsIdPrefix + aWidget.getPrototype().getID() + "-inst-" + aWidget.getID();
        else
            return;
        
        this._logger.debug("Remove widget: " + [toolbaritemId]);
        
        let browsers = typeof aBrowser != "undefined" ? [aBrowser] : this._getAllBrowsers();
        for each (let browser in browsers) {
            let browserDocument = browser.document;
            let toolbaritem = browserDocument.getElementById(toolbaritemId);
            if (!toolbaritem) {
                this._logger.warn("  Can't find toolbaritem. Widget not removed (" + [toolbaritemId] + ")");
                continue;
            }
            
            let toolbar = this._getToolbarForNode(toolbaritem);
            
            toolbaritem.parentNode.removeChild(toolbaritem);
            
            if (aPersist == true)
                this._persistToolbarSet(toolbar);
            
            this._logger.debug("  Widget removed (" + [toolbaritemId] + ")");
        }
    },
    
    insertWidget: function Overlay_insertToolbarItem(aProtoID, aBeforeElement, aPersist, aBrowser, settings) {
        if ((aBeforeElement instanceof Ci.nsIDOMElement) && !aBrowser)
            return [];
        
        if (aBeforeElement instanceof XB.WidgetLibrary.WidgetInstance)
            aBeforeElement = this._ItemsIdPrefix + aBeforeElement.getPrototype().getID() + "-inst-" + aBeforeElement.getID();
        
        function getBeforeElement(aToolbar) {
            let node;
            
            if (aBeforeElement) {
                if (aBeforeElement instanceof Ci.nsIDOMElement)
                    node = aBeforeElement;
                else
                    node = aToolbar.getElementsByAttribute("id", aBeforeElement)[0];
            }
            
            return node;
        }
        
        let widgetProto = this._application.widgetsRegistry.findProto(aProtoID);
        let widgetIDPrefix = this._ItemsIdPrefix + widgetProto.getID() + "-inst-";
        let iconPath = widgetProto.getIconPath();
        let widgetIconUrl = iconPath ? widgetProto.getLibrary().libPackage.getXBURL(iconPath) : "";
        let toolbaritemId = widgetIDPrefix + this._application.getNewWidgetInstanceID();
        
        let newItem = null;
        
        let browsers = typeof aBrowser != "undefined" ? [aBrowser] : this._getAllBrowsers();
        for each (let browser in browsers) {
            let browserDocument = browser.document;
            let toolbox = browserDocument.getElementById("navigator-toolbox");
            let toolbarPalette = toolbox.palette;
            let toolbar = browserDocument.getElementById(this._application.name + "-bar");
            if (toolbarPalette && toolbar) {
                newItem = this._makeToolbarItem(browserDocument, toolbaritemId, widgetProto.getName(), widgetIconUrl);
                if (settings) {
                    for (let settingName in settings) {
                        newItem.setAttribute("xbsetting-" + settingName, settings[settingName]);
                    }
                }
                
                toolbarPalette.appendChild(newItem);
                toolbar.insertItem(toolbaritemId, getBeforeElement(toolbar));
                
                if (aPersist === true)
                    this._persistToolbarSet(toolbar);
            }
        }
        
        return newItem ? [newItem.id, newItem] : [];
    },
    
    _cloneItemToPalette: function Overlay__cloneItemToPalette(aWidgetItem, aProtoID) {
        if (!(!!Ci.nsIWorker && aWidgetItem.parentNode))
            return;
        
        let palette;
        try {
            palette = aWidgetItem.ownerDocument.getElementById("navigator-toolbox").palette;
        } catch(e) {
            this._logger.error("cloneItemToPalette: can't find palette");
        }
        
        if (!palette)
            return;
        
        let widgetProto = this._application.widgetsRegistry.findProto(aProtoID);
        let iconPath = widgetProto.getIconPath();
        let widgetIconUrl = iconPath ? widgetProto.getLibrary().libPackage.getXBURL(iconPath) : "";
        
        let itemForPalette = this._makeToolbarItem(aWidgetItem.ownerDocument, aWidgetItem.id,
                                                   widgetProto.getName(), widgetIconUrl);
        palette.appendChild(itemForPalette);
    },
    
    _prefsCleanupDone: false,
    
    _cleanupPreferences: function Overlay__clearPreferences() {
        if (this._prefsCleanupDone)
            return;
        
        this._prefsCleanupDone = true;
        
        let prefsBranchPrefix = this._application.name + ".xbwidgets.";
        let prefSvc = Cc["@mozilla.org/preferences-service;1"]
                      .getService(Ci.nsIPrefService)
                      .getBranch(this._prefBranch)
                      .QueryInterface(Ci.nsIPrefBranch2);
        
        XB._base.application.widgetsRegistry.registeredProtoIDs.forEach(function(aProtoID) {
            let prefsToRemove = {};
            
            let currentsetIDs = this._currentSetIds[aProtoID] || [];
            let prefPrefix = prefsBranchPrefix + aProtoID + ".";
            prefSvc.getChildList(prefPrefix, {}).forEach(function(aPrefName) {
                let [,instId] = aPrefName.split(prefPrefix);
                if (instId) {
                    instId = instId.split(".")[0];
                    if (/^\d+$/.test(instId))
                        prefsToRemove[instId] = instId;
                }
            });
            
            for (let [instId] in Iterator(prefsToRemove)) {
                if (currentsetIDs.indexOf(instId) == -1) {
                    this._logger.info("clear pref: " + prefPrefix + instId);
                    try {
                        prefSvc.deleteBranch(prefPrefix + instId);
                    } catch(e) {
                        this._logger.warn(e);
                    }
                }
            }
        }, this);
    }
};