function SysUtils() {
    this._uconv = this._Cc["@mozilla.org/intl/scriptableunicodeconverter"].createInstance(
        this._Ci.nsIScriptableUnicodeConverter);
    this._uconv.charset = "UTF-8";
}

SysUtils.prototype = {
    readTextFile: function SysUtils_readTextFile(file) {
        var fileContents = "";
        
        var is = this._Cc["@mozilla.org/network/file-input-stream;1"].createInstance(this._Ci.nsIFileInputStream);
        is.init(file, 0x01, 0, is.CLOSE_ON_EOF);
        try {
            var sis = this._Cc["@mozilla.org/scriptableinputstream;1"].createInstance(this._Ci.nsIScriptableInputStream);
            sis.init(is);
            try {
                while(sis.available() > 0)
                    fileContents += this._uconv.ConvertToUnicode(sis.read(sis.available()));
            } finally {
                sis.close();
            }
        } finally {
            is.close();
        }
        
        return fileContents;
    },
    
    writeTextFile: function SysUtils_writeTextFile(file, text) {
        var fos = this._Cc["@mozilla.org/network/file-output-stream;1"].createInstance(this._Ci.nsIFileOutputStream);
        fos.init(file, 0x02 | 0x08 | 0x20, 0666, 0); 
        
        var converter = this._Cc["@mozilla.org/intl/converter-output-stream;1"]
            .createInstance(this._Ci.nsIConverterOutputStream);
        converter.init(fos, "UTF-8", 0, 0);
        converter.writeString(text);
        converter.close();
    },
    
    readXMLDocument: function SysUtils_readXMLDocument(file) {
        var docStream = this._Cc["@mozilla.org/network/file-input-stream;1"].createInstance(this._Ci.nsIFileInputStream);
        try {
            docStream.init(file, 0x01, 0600, 0);
            var XMLDoc = XB._Cc["@mozilla.org/xmlextras/domparser;1"]
                .createInstance(Ci.nsIDOMParser)
                    .parseFromStream(docStream, null, docStream.available(), "text/xml");
            if (XMLDoc.documentElement.localName == "parsererror")
                throw new Error("XML parse error");
            return XMLDoc;
        }
        finally {
            docStream.close();
        }
    },
    
    createFixupURIFromString: function SysUtils_createFixupURIFromString(aString) {
        let uri = null;
        if (aString) {
            let URIFixup = Components.classes["@mozilla.org/docshell/urifixup;1"]
                                     .getService(Components.interfaces.nsIURIFixup);
        
            try {
                uri = URIFixup.createFixupURI(aString, URIFixup.FIXUP_FLAG_NONE);
            } catch(e) {}
        }
        
        return uri;
    },
    
    trimString: function SysUtils_trimString(aString) {
        return aString.replace(/^\s\s*/, "").replace(/\s\s*$/, "");
    },
    
    strcmp: function SysUtils_strcmp(str1, str2) {
        if ( !(this.isString(str1) && this.isString(str2)) )
            throw new TypeError("Strings expected");
        return ( ( str1 == str2 ) ? 0 : ( ( str1 > str2 ) ? 1 : -1 ) );
    },
    
    strdup: function SysUtils_strdup(str, num) {
        var result = "";
        for (; num > 0; num--)
            result += str;
        return result;
    },
    
    isEmptyObject: function SysUtils_isEmptyObject(obj) {
        return this.objectsAreEqual(obj, this._emptyObj);
    },
    
    objectsAreEqual: function SysUtils_objectsAreEqual(obj1, obj2) {
        if (typeof obj1 != "object" || typeof obj2 != "object")
            throw new TypeError("Objects expected");
        
        var props1 = [], 
            props2 = [];
        
        for (let propName in obj1)
            props1.push(propName);
        
        for (let propName in obj2)
            props2.push(propName);
        
        if (props1.length != props2.length)
            return false;
        
        var prop1, prop2;
        for each (let propName in props1) {
            if ( !(propName in obj2) )
                return false;
            
            prop1 = obj1[propName];
            prop2 = obj2[propName];
            
            if ( (typeof prop1 == "object") && (typeof prop2 == "object") ) {
                if ( !this.objectsAreEqual(prop1, prop2) )
                    return false;
            }
            else {
                if (prop1 !== prop2)
                    return false;
            }
        }
        
        return true;
    },
    
    copyObj: function SysUtils_copyObj(src) {
        if ( !(src instanceof Object) )
            throw new TypeError("Object required");
        var result = new src.constructor(src);
        for (let propName in src) {
            let prop = src[propName];
            result[propName] = (prop instanceof Object)? this.copyObj(prop): prop;
        }
        return result;
    },
    
    isNumber: function SysUtils_isNumber(x) {
        return (typeof x == "number") || (x instanceof Number);
    },
    
    isString: function SysUtils_isString(x) {
        return (typeof x == "string") || (x instanceof String);
    },
    
    isBool: function SysUtils_isBool(x) {
        return (typeof x == "boolean") || (x instanceof Boolean);
    },
    
    _emptyObj: {},
    _Ci: Components.interfaces,
    _Cc: Components.classes,
    
    RegExp: {
        _escapeRe: /([.*+?^=!:${}()|[\]\/\\])/g,
        
        escape: function(aString) {
            return String(aString).replace(this._escapeRe, '\\$1');
        },
        
        createFromString: function(aString, aFlags) {
            return new RegExp(this.escape(aString), aFlags);
        }
    }
}

var sysutils = new SysUtils();