XB.Database = function() {
    this._connection = null;
}

XB.Database.prototype.open = function XBDB_open(fileName) {
    if (!fileName)
        throw new Error("Filename is required");
    
    this.close();
    
    var file = XB._Cc["@mozilla.org/file/directory_service;1"]
                   .getService(XB._Ci.nsIProperties)
                   .get("ProfD", XB._Ci.nsIFile);
    file.append(XB._base.application.name + "-xb");
    
    if (!(file.exists() && file.isDirectory()))
        file.create(XB._Ci.nsIFile.DIRECTORY_TYPE, 0755);
    
    file.append(fileName);
    
    var storage = XB._Cc["@mozilla.org/storage/service;1"].getService(XB._Ci.mozIStorageService);
    this._connection = storage.openDatabase(file);
}

XB.Database.prototype.close = function XBDB_close() {
    if (this._connection) {
        try {
            this._connection.close();
        } catch(e) {}
        
        this._connection = null;
    }
}

XB.Database.prototype._createStatement = function XBDB__createStatement(query, parameters) {
    let statement = this._connection.createStatement(query);
    if (statement && !("executeAsync" in statement)) {
        let wrappedStatement = XB._Cc["@mozilla.org/storage/statement-wrapper;1"].createInstance(XB._Ci.mozIStorageStatementWrapper);
        wrappedStatement.initialize(statement);
        statement = wrappedStatement;
    }
    
    if (parameters)
      for (let name in parameters)
        statement.params[name] = parameters[name];
    
    return statement;
}

XB.Database.prototype.exec = function XBDB_exec(query, parameters) {
    var statement,
        result = [];
    
    try {
        statement = this._createStatement(query, parameters);
        
        var columns = [];
        for (var i = statement.columnCount; i--;)
            columns.push(statement.getColumnName(i));
        
        while (statement.step()) {
            var row = {};
            for (var i = columns.length; i--;)
                row[columns[i]] = statement.row[columns[i]];

            result.push(row);
        }
    } catch(e) {
        XB._base.logger.error(e);
        XB._base.logger.debug(e.stack);
    } finally {
        if (statement)
            statement.reset();
    }
    
    return result;
}