XB.functions = {};

XB.functions.CN_setvar = XB._base.ProcNode.extend({
    expectedArgNames: ["name", "value"],
    
    _proc: function Fsetvar_proc() {
        var varName = this._argManager.getValByName("name", "String");
        var varNode = this._parentWidget.findVariable(varName);
        if (!varNode)
            throw new Error(XB._base.consts.ERR_DATA_UNDEFINED + " (" + varName + ")");
        var newValue = this._argManager.getValByName("value");
        varNode.setValue(newValue);
    }
});

XB.functions.Arithmetical = XB._base.FuncNode.extend({
    _calculate: function FArithmetical_calculate() {
        return this._applyOp(this._operation);
    },
    
    _applyOp: function FArithmetical_applyOp(op) {
        var result = NaN;
        for (let valIndex = 0, len = this._argManager.argsCount; valIndex < len; valIndex++) {
            var num = this._argManager.getValByIndex(valIndex, "Number");
            if (isNaN(result))
                result = num;
            else
                result = op(result, num);
        }
        return result;
    }
});

XB.functions.CN_add = XB.functions.Arithmetical.extend({
    _operation: function Fadd_operation(p1, p2) {
        return p1 + p2;
    }
});

XB.functions.CN_sub = XB.functions.Arithmetical.extend({
    _operation: function Fsub_operation(p1, p2) {
        return p1 - p2;
    }
});

XB.functions.CN_mul = XB.functions.Arithmetical.extend({
    _operation: function Fmul_operation(p1, p2) {
        return p1 * p2;
    }
});

XB.functions.CN_div = XB.functions.Arithmetical.extend({
    _operation: function Fdiv_operation(p1, p2) {
        return p1 / p2;
    }
});

XB.functions.CN_mod = XB.functions.Arithmetical.extend({
    _operation: function Fmod_operation(p1, p2) {
        return p1 % p2;
    }
});

XB.functions["CN_format-number"] = XB._base.FuncNode.extend({
    expectedArgNames: ["value", "precision"],
    
    _calculate: function FFormatNumber_calculate() {
        var num = this._argManager.getValByName("value", "Number");
        var precision = parseInt( this._argManager.getValByName("precision", "Number") );
        if (precision < 0)
            throw new RangeError("Precision must be positive");
        
        var order = Math.pow(10, precision);
        var rounded = Math.round(Math.abs(num) * order);
        var result = ((num < 0)? "-" : "") + parseInt(rounded / order).toLocaleString();
        if (precision > 0) {
            var fracStr = rounded.toString().substr(-precision);
            fracStr = sysutils.strdup("0", precision - fracStr.length) + fracStr;
            result += XB._base.consts.STR_LOCALE_DECIMAL_SEPARATOR + fracStr;
        }
        
        return result;
    }
});

XB.functions.Periodic = XB._base.FuncNode.extend({
    expectedArgNames: ["expire"],
    
    constructor: function FPeriodic_constructor() {
        this.base.apply(this, arguments);
        this._timer = XB._Cc["@mozilla.org/timer;1"].createInstance(XB._Ci.nsITimer);
    },
    
    _resetTimer: function FPeriodic_resetTimer() {
        var expireTime = this._argManager.getValByNameDef("expire", "Number", Number.POSITIVE_INFINITY);
        if (expireTime == Number.POSITIVE_INFINITY) {
            this._timerSet = true;
            return;
        }
        
        if (expireTime <= 0)
            throw new RangeError(this._consts.ERR_INVALID_INTERVAL);
        expireTime = Math.max(expireTime, XB.functions.Periodic.MIN_INTERVAL);
        
        this._timer.initWithCallback(this, expireTime * 1000, this._timer.TYPE_REPEATING_SLACK);
        this._timerSet = true;
        this._timerInterval = expireTime;
    },
    
    _cancelTimer: function FPeriodic_cancelTimer() {
        this._timer.cancel();
        this._timerSet = false;
    },
    
    _calculate: function FPeriodic_calculate(changedArgs) {
        if (this._hasSubscribers() && (!this._timerSet || this._argManager.argInArray("expire", changedArgs)))
            this._resetTimer();
        else
            this._cancelTimer();
        return this._storedValue;
    },
    
    _notNeeded: function FPeriodic_notNeeded() {
        this._cancelTimer();
    },
    
    _timer: null,
    _timerInterval: undefined,
    _timerSet: false
}, {
    MIN_INTERVAL: 1
});

XB.functions.CN_timestamp = XB.functions.Periodic.extend({
    notify: function Ftimestamp_notify() {
        try {
            this._setNewVal( parseInt((new Date()).getTime() / 1000) );
            this._notifyDeps();
        }
        catch (e) {
            XB._base.logger.error("CN_timestamp.notify() failed. " + e.message);
        }
    },
    
    _calculate: function Ftimestamp_calculate() {
        this.base();
        return parseInt((new Date()).getTime() / 1000);
    }
});

XB.functions.CN_meander = XB.functions.Periodic.extend({
    notify: function Fmeander_notify() {
        try {
            this._setNewVal(!this._storedValue);
            this._notifyDeps();
        }
        catch (e) {
            XB._base.logger.error("CN_meander.notify() failed. " + e.message);
        }
    }
});

XB.functions["CN_time-passed"] = XB._base.FuncNode.extend({
    expectedArgNames: ["timestamp"],
    
    constructor: function Ftimepassed_constructor() {
        this.base.apply(this, arguments);
        this._timer = XB._Cc["@mozilla.org/timer;1"].createInstance(XB._Ci.nsITimer);
    },
    
    _calculate: function Ftimepassed_calculate(changedArgs) {
        var moreToWait = this._argManager.getValByName("timestamp", "Number") - parseInt((new Date()).getTime() / 1000);
        if (moreToWait > 0)
            this._timer.initWithCallback(this, moreToWait * 1000, this._timer.TYPE_ONE_SHOT);
        return moreToWait <= 0;
    },
    
    notify: function Fwait_notify() {
        try {
            if (this._setNewVal(true))
                this._notifyDeps();
        }
        catch (e) {
            XB._base.logger.error("CN_time-passed.notify() failed. " + e.message);
        }
    },
    
    _timer: null
});

XB.functions.IPassingValue = Base.extend({
    _setNewVal: function Fvalueof_setNewVal(newVal) {
        this._storedValue = newVal;
        return true;
    }
});

XB.functions.CN_if = XB._base.FuncNode.extend({
    expectedArgNames: ["condition", "ontrue", "onfalse"],
    
    _calculate: function Fif_calculate() {
        if ( this._argManager.getValByName("condition", "Bool") )
            return this._argManager.getValByNameDef("ontrue", undefined, XB.types.empty);
        return this._argManager.getValByNameDef("onfalse", undefined, XB.types.empty);
    }
});

XB.functions.CN_if.implement(XB.functions.IPassingValue);

XB.functions.CN_concat = XB._base.FuncNode.extend({
    _calculate: function Fconcat_calculate() {
        if (this._argManager.argsCount < 2) {
            return (this._argManager.argsCount == 1 ? this._argManager.getValByIndex(0) : XB.types.empty );
        }
        
        var resultIsXML = false;
        var argXValues = [];
        
        for (let valIndex = 0, len = this._argManager.argsCount; valIndex < len; valIndex++) {
            var xv = this._argManager.getValByIndex(valIndex);
            argXValues.push(xv);
            resultIsXML = resultIsXML || XB._base.runtime.isXML(xv);
        }
        
        if (!resultIsXML) {
            var result = "";
            for (let i = 0, len = argXValues.length; i < len; i++) {
                result += XB._base.runtime.xToString(argXValues[i]);
            }
            return result;
        }
        else {
            var result = new XB.types.XML();
            var xv;
            for (let i = 0, len = argXValues.length; i < len; i++) {
                xv = argXValues[i];
                if (XB._base.runtime.isXML(xv))
                    result.appendCopy(xv);
                else {
                    var node = XB._base.runtime.tempXMLDoc.createTextNode(XB._base.runtime.xToString(xv));
                    result.appendChild(node);
                }
            }
            return result;
        }
    }
});

XB.functions.CN_xpath = XB._base.FuncNode.extend({
    expectedArgNames: ["expression", "source"],
    
    _calculate: function Fxpath_calculate() {
        var expr = this._argManager.getValByName("expression", "String");
        var src = this._argManager.getValByName("source");
        if (!XB._base.runtime.isXML(src))
            return new XB.types.Exception(this.getEffectiveID(), XB.types.Exception.types.E_TYPE);
        return src.query(expr);
    }
});

XB.functions.CN_xslt = XB._base.FuncNode.extend({
    expectedArgNames: ["stylesheet", "src-xml"],
    
    _calculate: function Fxslt_calculate() {
        var stylesheetPath = this._argManager.getValByName("stylesheet", "String");
        var stylesheetFile = this._parentWidget.getPrototype().getLibrary().libPackage.getFile(stylesheetPath);
        var stylesheet = sysutils.readXMLDocument(stylesheetFile);
        var src = this._argManager.getValByName("src-xml");
        if (!XB._base.runtime.isXML(src))
            return new XB.types.Exception(this.getEffectiveID(), XB.types.Exception.types.E_TYPE);
        return src.transform(stylesheet);
    }
});

XB.functions["CN_value-of"] = XB._base.FuncNode.extend({
    expectedArgNames: ["name"],
    
    _calculate: function Fvalueof_calculate(changedArgs) {
        if (!this._refNode || this._argManager.argInArray("name", changedArgs))
            this._findRefNode();
        return this._refNode.getValue(this._hasSubscribers()? this: null);
    },
    
    _findRefNode: function Fvalueof_findRefNode() {
        var refNode;
        var refName = this._argManager.getValByName("name", "String");
        
        refNode = this._parentWidget.findData(refName);
        
        if (!refNode)
            refNode = this._parentWidget.findVariable(refName);
        
        if (!refNode)
            refNode = this._parentWidget.findSetting(refName);
        
        if (refNode == undefined)
            throw new Error(XB._base.consts.ERR_DATA_UNDEFINED + " (" + refName + ")");
        
        if (this._refNode != refNode) {
            if (this._refNode)
                this._refNode.unsubscribe(this);
            this._refNode = refNode;
        }
    },
    
    _refNode: null
});

XB.functions["CN_value-of"].implement(XB.functions.IPassingValue);

XB.functions.Comparision = XB._base.FuncNode.extend({
    expectedArgNames: ["lhs", "rhs"],
    
    _calculate: function FComparision_calculate(changedArgs) {
        var mode = this._argManager.getValByNameDef("mode", "String", "default");
        var strict = (mode == "strict");
        var left = this._argManager.getValByName("lhs");
        var right = this._argManager.getValByName("rhs");
        return this._op( XB._base.runtime.compareValues(left, right, strict) );
    }
});

XB.functions["CN_is-empty"] = XB._base.FuncNode.extend({
    _calculate: function Fisempty_calculate(changedArgs) {
        var what = this._argManager.getValByIndex(0);
        return XB._base.runtime.compareValues(what, XB.types.empty, true) == 0;
    }
});

XB.functions.CN_eq = XB.functions.Comparision.extend({
    _op: function Feq_op(compResult) {
        return compResult == 0;
    }
});

XB.functions.CN_equal = XB.functions.CN_eq;

XB.functions.CN_neq = XB.functions.CN_eq.extend({
    _calculate: function Fneq_calculate() {
        return !this.base();
    }
});

XB.functions["CN_not-equal"] = XB.functions.CN_neq;

XB.functions.CN_lt = XB.functions.Comparision.extend({
    _op: function Flt_op(compResult) {
        return compResult < 0;
    }
});

XB.functions.CN_lte = XB.functions.Comparision.extend({
    _op: function Flte_op(compResult) {
        return compResult <= 0;
    }
});

XB.functions.CN_gt = XB.functions.Comparision.extend({
    _op: function Fgt_op(compResult) {
        return compResult > 0;
    }
});

XB.functions.CN_gte = XB.functions.Comparision.extend({
    _op: function Fgte_op(compResult) {
        return compResult >= 0;
    }
});

XB.functions.CN_not = XB._base.FuncNode.extend({
    _calculate: function Fnot_calculate() {
        return !(this._argManager.getValByIndex(0, "Bool"));
    }
});

XB.functions.CN_or = XB._base.FuncNode.extend({
    _calculate: function For_calculate() {
        for (let i = 0, len = this._argManager.argsCount; i < len; i++) {
            var val = this._argManager.getValByIndex(i);
            if (XB._base.runtime.xToBool(val))
                return true;
        }
        return false;
    }
});

XB.functions.CN_coalesce = XB._base.FuncNode.extend({
    _calculate: function Fcoalesce_calculate() {
        for (let i = 0, len = this._argManager.argsCount; i < len; i++) {
            var val = this._argManager.getValByIndex(i);
            if (val != XB.types.empty)
                return val;
        }
        return XB.types.empty;
    }
});

XB.functions.CN_and = XB._base.FuncNode.extend({
    _calculate: function Fand_calculate() {
        for (let i = 0, len = this._argManager.argsCount; i < len; i++) {
            var val = this._argManager.getValByIndex(i);
            if (!XB._base.runtime.xToBool(val))
                return false;
        }
        return true;
    }
});

XB.functions.CN_try = XB._base.FuncNode.extend({
    expectedArgNames: ["expression"],
    
    _calculate: function Ftry_calculate() {
        var mainArg = this._argManager.findNodeByName("expression");
        var mainVal = mainArg.getValue(this._hasSubscribers()? this: null);
        
        if (mainVal instanceof XB.types.Exception) {
            XB._base.logger.trace("Cought exception of type " + mainVal.type);
            var excHandler = this._argManager.findNodeByName(mainVal.type) ||
                             this._argManager.findNodeByName(this._consts.STR_ALL_EXCEPTIONS);
            
            if (excHandler) {
                var handlerVal = excHandler.getValue(this._hasSubscribers()? this: null);
                if (handlerVal instanceof XB.types.Exception) {
                    XB._base.logger.trace("Exception rethrown with type " + handlerVal.type);
                    switch (handlerVal.type) {
                        case XB.types.Exception.types.E_RETHROW:
                            handlerVal = mainVal;
                            break;
                        case XB.types.Exception.types.E_LASTVALUE:
                            handlerVal = this._storedValue;
                            break;
                    }
                }
                XB._base.logger.debug("Exception handler returned " + handlerVal);
                return handlerVal;
            }
            return mainVal;
        }
        return mainVal;
    },
    
    _consts: {
        STR_ALL_EXCEPTIONS: "..."
    }
});

XB.functions.CN_throw = XB._base.FuncNode.extend({
    expectedArgNames: ["type"],
    
    _calculate: function Fthrow_calculate() {
        var eType = this._argManager.getValByNameDef("type", "String", XB.types.Exception.types.E_RETHROW);
        
        var Exception = new XB.types.Exception(this.getEffectiveID(), eType);
        var excPropVal;
        var excPropsNames = this._argManager.getArgsNames();
        for each(let propName in excPropsNames) {
            if (propName == "type")
                continue;
            excPropVal = this._argManager.getValByName(propName);
            Exception[propName] = excPropVal;
        }
        
        return Exception;
    }
});

XB.functions.CN_lastvalue = XB._base.FuncNode.extend({
    _calculate: function Flastvalue_calculate() {
        var lastValException = new XB.types.Exception(this.getEffectiveID(), XB.types.Exception.types.E_LASTVALUE);
        return lastValException;
    }
});

XB.functions.CN_optional = XB._base.FuncNode.extend({
    _calculate: function Foptional_calculate() {
    }
});

XB.functions["CN_window-title"] = XB._base.FuncNode.extend({
    _calculate: function Fwindowtitle_calculate() {
        if (!this._mainWindow) {
            this._mainWindow = this._parentWidget.getWindowEngine().window;
            if (this._hasSubscribers())
                this._mainWindow.gBrowser.tabContainer.addEventListener("TabSelect", this, false);
            this._updateCurrContent();
        }
        
        return this._getTitle();
    },
    
    handleEvent: function Fwindowtitle_handleEvent(event) {
        try {
            switch (event.type) {
                case "TabSelect":
                    this._updateCurrContent();
                case "load":
                    if (this._setNewVal( this._getTitle() ))
                        this._notifyDeps();
                    break;
            }
        }
        catch (e) {
            XB._base.logger.error("window-title::handleEvent failed. " + XB._base.formatException(e));
        }
    },
    
    _notNeeded: function Fwindowtitle_notNeeded() {
        this._mainWindow.gBrowser.tabContainer.removeEventListener("TabSelect", this, false);
    },
    
    _mainWindow: null,
    _currWatchedContent: null,
    
    _updateCurrContent: function Fwindowtitle_finalize() {
        if (this._currWatchedContent)
            this._currWatchedContent.removeEventListener("load", this, false);
        this._currWatchedContent = this._mainWindow.content;
    },
    
    _getTitle: function Fwindowtitle_getTitle() {
        return this._currWatchedContent.document.title;
    }
});

XB.functions.CN_request = XB._base.FuncNode.extend({
    expectedArgNames: ["url", "update", "expire", "format", "valid-status", "valid-xpath"],
    
    _calculate: function Frequest_calculate(changedArgs) {
        var format;
        var formatStr = this._argManager.getValByNameDef("format", "String", "text");
        switch (formatStr) {
            case "text":
                format = XB.types.RequestData.Format.FMT_TEXT;
                break;
            case "xml":
                format = XB.types.RequestData.Format.FMT_XML;
                break;
            case "json":
                format = XB.types.RequestData.Format.FMT_JSON;
                break;
            default:
                throw new Error("Unknown format specification: " + formatStr);
        }
        
        var validStatusRange = {start: 100, end: 299};
        var statusesExpr = this._argManager.getValByNameDef("valid-status", "String", "");
        if (statusesExpr) {
            var matches = statusesExpr.match(/^(\d+)?\.\.(\d+)?$/);
            if (matches != null) {
                validStatusRange.start = matches[1] || 100;
                validStatusRange.end = matches[2] || 299;
            }
            else {
                XB._base.logger.warn("Could not parse request status validation expression (" + statusesExpr + ")");
            }
        }
        var updateInterval = this._argManager.getValByNameDef("update", "Number", 0);
        if (updateInterval == 0)
            updateInterval = Number.POSITIVE_INFINITY;
        
        var reqData = new XB.types.RequestData(this._argManager.getValByName("url", "String"),
                                               "GET",
                                               updateInterval,
                                               this._argManager.getValByNameDef("expire", "Number", 0),
                                               format,
                                               validStatusRange,
                                               this._argManager.getValByNameDef("valid-xpath", "String"));
        return reqData;
    }
});


XB.functions.NetworkData = XB._base.FuncNode.extend({
    expectedArgNames: ["request"],
    
    update: function FNetworkData_update(netResource, netData) {
        if (netResource !== this._resource) {
            XB._base.logger.warn("Update notification from a martian network resource! " + this._getHumanReadableID());
            return;
        }
        if (this._setNewVal(this._processResData(netData)))
            this._notifyDeps();
    },
    
    _calculate: function FNetworkData_calculate(changedArgs) {
        var reqData = this._argManager.getValByName("request", "RequestData");
        var netResource = XB.network.getResource(reqData);
        if (this._resource != netResource) {
            if (this._resource)
                try {
                    this._resource.unsubscribe(this, this._dataType);
                }
                catch (e) {
                    XB._base.logger.error("Could not unsubscribe from a network resource. " +
                                          XB._base.formatException(e));
                    XB._base.logger.debug(e.stack);
                }
            this._resource = netResource;
        }
        
        var rawData = this._resource.requestData(this, this._dataType);
        XB._base.logger.debug("FNetworkData_calculate raw data is " + rawData);
        return this._processResData(rawData);
    },
    
    _resource: null,
    
    _notNeeded: function FNetworkData_notNeeded() {
        if (this._resource) {
            this._resource.unsubscribe(this, this._dataType);
            delete this._resource;
        }
    }
});

XB.functions.CN_content = XB.functions.NetworkData.extend({
    _dataType: XB.network.ResourceDataType.NRDT_CONTENT,
    
    _processResData: function Fcontent_processResData(content) {
        if (!content)
            return XB.types.empty;
        
        if (content instanceof XB._Ci.nsIDOMDocument) {
            return new XB.types.XML(content.documentElement);
        }
        
        if (!sysutils.isString(content))
            throw new TypeError("Network resource sent not an XML document, nor a String");
        
        return content;
    }
});

XB.functions.CN_header = XB.functions.NetworkData.extend({
    expectedArgNames: ["request", "name"],
    
    _dataType: XB.network.ResourceDataType.NRDT_HEADERS,
    
    _calculate: function Fheader_calculate(changedArgs) {
        if (this._argManager.argInArray("name", changedArgs) || !this._headerName) {
            this._headerName = this._argManager.getValByName("name", "String");
        }
        return this.base(changedArgs);
    },
    
    _processResData: function Fheader_processResData(resHeaders) {
        return XB.types.empty;
    },
    
    _headerName: undefined
});

XB.functions.CN_finished = XB._base.FuncNode.extend({
    expectedArgNames: ["requestid"],
    
    onRequestFinished: function Ffinished_onRequestFinished(requestID) {
        if (this._requestID == requestID)
            if (this._setNewVal(true))
                this._notifyDeps();
        this._requestID = undefined;
    },
    
    _calculate: function Ffinished_calculate(changedArgs) {
        var requestID = this._argManager.getValByName("requestid", "Number");
        if (XB.network.requestFinished(requestID)) return true;
        
        XB.network.subscribeToRequestFinish(requestID, this);
        this._requestID = requestID;
        return false;
    },
    
    _requestID: undefined
});

XB.functions.CN_update = XB._base.ProcNode.extend({
    expectedArgNames: ["request"],
    
    _proc: function Fupdate_proc() {
        var requestData = this._argManager.getValByName("request", "RequestData");
        var netResource = XB.network.getResource(requestData);
        return netResource.update(true);
    }
});

XB.functions.CN_file = XB._base.FuncNode.extend({
    expectedArgNames: ["path", "type"],
    
    _calculate: function Ffile_calculate() {
        var path = this._argManager.getValByName("path", "String");
        var type = this._argManager.getValByNameDef("type", "String", "text");
        var file = this._parentWidget.getPrototype().getLibrary().libPackage.getFile(path);
        
        switch (type) {
            case "xml":
                return new XB.types.XML( sysutils.readXMLDocument(file).documentElement );
            case "text":
                return sysutils.readTextFile(file);
            default:
                throw new Error("Unsupported file output type");
        }
    }
});

XB.functions["CN_copy-to-clipboard"] = XB._base.ProcNode.extend({
    constructor: function FCopyToClipboard_constructor() {
        this.base.apply(this, arguments);
        this._helper = XB._Cc["@mozilla.org/widget/clipboardhelper;1"].getService(XB._Ci.nsIClipboardHelper);
    },
    
    _helper: null,
    
    _proc: function FcopyToClipboard_proc() {
        this._helper.copyString(this._argManager.getValByIndex(0, "String"));
    }
});

XB.functions.CN_navigate = XB._base.ProcNode.extend({
    expectedArgNames: ["url", "target"],
    
    _proc: function Fnavigate_proc() {
        var url = this._argManager.getValByName("url", "String");
        var target = this._argManager.getValByNameDef("target", "String", "");
        var yandexAction = this._argManager.getValByNameDef("yandex-action", "String", "");
        var wnd = this._parentWidget.getWindowEngine().window;
        XB._base.application.navigate(wnd, url, target, yandexAction);
    }
});
