XB.Preset = function() {
    XB._base.logger.info("Creating XB Preset");
};

XB.Preset.prototype = {
    get widgetsInfo() {
        return sysutils.copyObj( this._widgetsInfo );
    },
    
    clear: function() {
        this._widgetsInfo = [];
    },
    
    loadFromFile: function(presetFile) {
        XB._base.logger.debug("Loading preset from " + presetFile.path);
        this.loadFromDocument( sysutils.readXMLDocument(presetFile) );
    },
    
    loadFromDocument: function(XMLDocument) {
        this.clear();
        if (XMLDocument.documentElement.localName != this._consts.STR_PRESET_NODE_NAME)
            throw new TypeError(XB._base.consts.ERR_INVALID_DOC_STRUC);
        this._parsePreset(XMLDocument.documentElement);
        XB._base.logger.debug("Preset loaded");
    },
    
    _consts: {
        STR_PRESET_NODE_NAME: "preset",
        STR_SETTING_ELEMENT_NAME: "setting",
        
        WARN_WIDGET_IGNORED: "Widget was ignored because of syntax errors",
        WARN_SETTING_IGNORED: "Widget setting was ignored because of syntax errors"
    },
    _widgetsInfo: undefined,
    
    _parsePreset: function(presetElement) {
        for (let elementIndex = 0, len = presetElement.childNodes.length; elementIndex < len; elementIndex++) {
            var widgetElement = presetElement.childNodes[elementIndex];
            if (widgetElement.nodeType != widgetElement.ELEMENT_NODE ||
                widgetElement.localName != XB._base.consts.STR_WIDGET_ELEMENT_NAME) continue;
            var protoID = widgetElement.getAttribute("id");
            if (!protoID) {
                XB._base.logger.warn(this._consts.WARN_WIDGET_IGNORED);
                continue;
            }
            var widgetInfo = {
                protoID: protoID,
                settings: {}
            };
            
            for (let settingIndex = widgetElement.childNodes.length; settingIndex--; ) {
                var settingElement = widgetElement.childNodes[settingIndex];
                if (settingElement.nodeType != settingElement.ELEMENT_NODE ||
                    settingElement.localName != this._consts.STR_SETTING_ELEMENT_NAME) continue;
                
                var settingName = settingElement.getAttribute("name");
                if (!settingName) {
                    XB._base.logger.warn(this._consts.WARN_SETTING_IGNORED);
                    continue;
                }
                widgetInfo.settings[settingName] = settingElement.textContent;
            }
            
            this._widgetsInfo.push(widgetInfo);
        }
    }
};