XB.types = {};

XB.types.empty = null;

XB.types.XML = function XML_constructor() {
    this._frag = XB._base.runtime.tempXMLDoc.createDocumentFragment();
    this._rootNode = this._frag.appendChild(XB._base.runtime.tempXMLDoc.createElement("xml"));
    if (!this._rootNode)
        throw new Error("Couldn't create root node");
    this.appendCopy.apply(this, arguments);
};

XB.types.XML.prototype = {
    appendCopy: function XML_appendCopy() {
        for (let argIndex = 0, argsLen = arguments.length; argIndex < argsLen; argIndex++) {
            var arg = arguments[argIndex];
            if (arg instanceof Array) {
                this.appendCopy.apply(this, arg);
                return;
            }
            if (arg instanceof XB._Ci.nsIDOMNode)
                this._appendAnyNode(arg);
            else
                if (arg instanceof XB._Ci.nsIDOMNodeList || arg instanceof this.constructor) {
                    for (let nodeIndex = 0, nodesLen = arg.length; nodeIndex < nodesLen; nodeIndex++) {
                        this._appendAnyNode(arg.item(nodeIndex));
                    }
                }
                else
                    throw new TypeError(XB._base.consts.ERR_DOM_NODES_EXPECTED);
        }
    },
    
    appendChild: function XML_moveFrom(node) {
        this._rootNode.appendChild(node);
    },
    
    item: function XML_Item(at) {
        return this._rootNode.childNodes[at];
    },
    
    get childNodes() {
        return this._rootNode.childNodes;
    },
    
    get length() {
        return this._rootNode.childNodes.length;
    },
    
    dispose: function XML_Dispose() {
        if (this._frag)
            this._frag.removeChild(this._rootNode);
        else
            XB._base.logger.warn("Attemp to dispose an already disposed XML object.");
        this._rootNode = null;
        this._frag = null;
    },
    
    serialize: function XML_Serialize() {
        var result = "";
        for (let nodeIndex = 0, len = this.length; nodeIndex < len; nodeIndex++) {
            result += XB._base.runtime.serializeXML(this.item(nodeIndex));
        }
        return result;
    },
    
    get textContent() {
        return this._rootNode.textContent;
    },
    
    equalsTo: function XML_EqualsTo(other) {
        if ( !(other instanceof this.constructor) )
            throw new TypeError("XML type expected");
        
        if (!other._rootNode)
            return false;
        return this._rootNode.isEqualNode(other._rootNode);
    },
    
    query: function XML_Query(expr) {
        if (XB._base.logger.level <= 10) {
            XB._base.logger.trace("Query from:\n" + this.serialize().substr(0, 400));
        }
        var queryResult = XB._base.runtime.queryXMLDoc(expr, this._rootNode);
        return XB.types.XML.handleQueryResult(queryResult);
    },
    
    transform: function XML_Transform(stylesheet) {
        var newFragment = XB._base.runtime.transformXML(this._rootNode, stylesheet);
        var newXML = new XB.types.XML();
        newXML.appendChild(newFragment);
        return newXML;
    },
    
    _frag: null,
    _rootNode: null,
    
    _appendAnyNode: function XML_appendAnyNode(node) {
        var clone = (node.ownerDocument == this._rootNode.ownerDocument)?
            node.cloneNode(true) :
            this._rootNode.ownerDocument.importNode(node, true);
        this._rootNode.appendChild(clone);
    }
};

XB.types.XML.handleQueryResult = function XBXML_handleQueryResult(queryResult) {
    if (queryResult instanceof Array) {
        XB._base.logger.debug("XML query returned an array of nodes " + queryResult.length);
        if (!queryResult.length)
            return XB.types.empty;
        queryResult = new XB.types.XML(queryResult);
        XB._base.logger.debug("Constructed a new XML " + queryResult.length);
    }
    
    return queryResult;
};

XB.types.Exception = function XBException_constructor(srcNodeUid, eType) {
    this._type = eType || this._type;
    this._srcNodeUid = srcNodeUid;
};

XB.types.Exception.types = {
    E_GENERIC: "Exception",
    E_SYNTAX: "Syntax",
    E_TYPE: "Type",
    E_RUNTIME: "Runtime",
    E_RETHROW: "Rethrow",
    E_LASTVALUE: "LastValue"
};

XB.types.Exception.prototype = {
    get type() {
        return this._type;
    },
    
    get srcNodeUid() {
        return this._srcNodeUid;
    },
    
    equalsTo: function XBException_equalsTo(other) {
        return sysutils.objectsAreEqual(this, other);
    },
    
    _type: XB.types.Exception.types.E_GENERIC,
    _srcNodeUid: undefined
};

XB.types.RequestData = function RequestData_constructor(url, method, update, expire, format, validStatusRange, validXpath) {
    XB._base.runtime.ensureValueTypeIs(url, "String");
    this._url = url;
    
    XB._base.runtime.ensureValueTypeIs(method, "String");
    this._method = method;
    
    XB._base.runtime.ensureValueTypeIs(update, "Number");
    if (update < 1)
        throw new RangeError("Invalid update interval " + update);
    this._updateInterval = update;
    
    XB._base.runtime.ensureValueTypeIs(expire, "Number");
    if (expire < 1)
        throw new RangeError("Invalid expiration time " + expire);
    this._expirationInterval = expire;
    
    XB._base.runtime.ensureValueTypeIs(format, "Number");
    if (format < XB.types.RequestData.Format.FMT_TEXT || format > XB.types.RequestData.Format.FMT_JSON)
        throw new RangeError("Unknown format type");
    this._format = format;
    
    if (validStatusRange && sysutils.isNumber(validStatusRange.start) && sysutils.isNumber(validStatusRange.end))
        this._statusRange = validStatusRange;
    
    if (validXpath) {
        XB._base.runtime.ensureValueTypeIs(validXpath, "String");
        this._checkXpathExpr = validXpath;
    }
};

XB.types.RequestData.Format = {
    FMT_TEXT: 0,
    FMT_XML: 1,
    FMT_JSON: 2
};

XB.types.RequestData.prototype = {
    constructor: XB.types.RequestData,
    
    get url() {
        return this._url;
    },
    
    get method() {
        return this._method;
    },
    
    get updateInterval() {
        return this._updateInterval;
    },
    
    get expirationInterval() {
        return this._expirationInterval;
    },
    
    get format() {
        return this._format;
    },
    
    get statusRange() {
        return this._statusRange;
    },
    
    get xpathExpression() {
        return this._checkXpathExpr;
    },
    
    get hash() {
        return [this._url, this._method, this.updateInterval, this.expirationInterval, this._format,
                this._statusRange.start, this._statusRange.end, this._checkXpathExpr].join("#");
    },
    
    _url: undefined,
    _method: "GET",
    _updateInterval: 0,
    _expirationInterval: 0,
    _format: XB.types.RequestData.Format.FMT_TEXT,
    _statusRange: null,
    _checkXpathExpr: undefined
};
