function ValWidgetMap() {
    this._uiArray = [];
    this._valHash = { __proto__: null };
}

ValWidgetMap.prototype = {
    put: function VWM_put(aValUid, aWidgetId, aUiElement, aXbElement) {
        aValUid = aValUid || (new Date().getTime() + Math.random());
        
        let id = [aWidgetId, aValUid].join(":");
        
        let uiElementRef = this._getReference(aUiElement);
        
        if (!this._valHash[id]) {
            this._valHash[id] = {
                uiElements: [uiElementRef],
                xbElement: aXbElement
            }
        } else if (this._valHash[id].uiElements.indexOf(uiElementRef) == -1) {
            this._valHash[id].uiElements.push(uiElementRef);
        }
        
        let uiArrayElement = this._getUiArrayElement(aUiElement);
        
        if (!uiArrayElement) {
            this._uiArray.push({
                uiElement: uiElementRef,
                widgetId: aWidgetId,
                uids: [id]
            });
        } else if (uiArrayElement.uids.indexOf(id) == -1) {
            uiArrayElement.uids.push(id);
        }
    },
    
    get: function VWM_get(aWidgetId, aValUid) {
        let id = [aWidgetId, aValUid].join(":");
        let val = this._valHash[id];
        if (val) {
            let uiElements = [];
            let uiElementsRef = val.uiElements;
            
            for (let i=0, len = uiElementsRef.length; i < len; i++) {
                let uiElement = this._getReferent(uiElementsRef[i]);
                if (uiElement) {
                    uiElements.push(uiElement);
                } else {
                }
            }
            
            return [uiElements, val.xbElement];
        }
        
        return [];
    },
    getByUiElement: function VWM_getByUiElement(aUiElement) {
        let uiArrayElement = this._getUiArrayElement(aUiElement);
        
        if (uiArrayElement) {
            let xbElements = [];
            let uids = uiArrayElement.uids;
            let widgetId = uiArrayElement.widgetId;
            
            for (let i=0, len = uids.length; i < len; i++) {
                let val = this._valHash[uids[i]];
                xbElements.push(val.xbElement);
            }
            
            return [widgetId, xbElements];
        }
        
        return [];
    },
    
    removeUiElement: function VWM_removeUiElement(aUiElement) {
        let uiArrayElement = this._getUiArrayElement(aUiElement);
        
        if (uiArrayElement) {
            let uiElementRef = this._getReference(aUiElement);
            
            let uids = uiArrayElement.uids;
            for (let i=0, len = uids.length; i < len; i++) {
                let val = this._valHash[uids[i]];
                val.uiElements.splice(val.uiElements.indexOf(uiElementRef), 1);
            }
            
            this._uiArray.splice(this._uiArray.indexOf(uiArrayElement), 1);
            
            return true;
        }
        
        return false;
    },
    
    _getUiArrayElement: function VWM__getUiArrayElement(aUiElement) {
        let uiArrayElement;
        
        this._uiArray.some(function(el) {
            return this._getReferent(el.uiElement) === aUiElement ? !!(uiArrayElement = el) : false;
        }, this);
        
        return uiArrayElement;
    },
    
    _getReference: function VWM__getReference(aUiElement) {
        return (aUiElement instanceof Ci.nsISupportsWeakReference) ? aUiElement.GetWeakReference() : null;
    },
    
    _getReferent: function VWM__getReferent(aWeakRef) {
        try {
            return aWeakRef.QueryReferent(Ci.nsISupports);
        } catch (e) {}
        
        return null;
    }
}

XB.XULBuilder = function(windowEngine, overlayController) {
    this._xid = XB._base.genXULBuilderID();
    this._logger = XB._base.application.core.Log4Moz.repository.getLogger(XB._base.loggersRoot + ".XUL" + this._xid);
    this._wndEngine = windowEngine;
    this._overlayController = overlayController;
    this._valWidgetMap = new ValWidgetMap();
    this._fullWidgetItemIDPattern = new RegExp(XB._base.application.name + "\\.cb\\-(.+)\\-inst\\-(.+)");
}

XB.XULBuilder.prototype = {
    CSS_ATTR_NAMES: {
        __proto__: null,
        "weight": "fontWeight",
        "color": "color",
        "h-align": "textAlign"
    },
    
    finalize: function GXB_finalize() {
        this._logger.debug("Finalize");
        this._valWidgetMap = null;
        this._overlayController = null;
        this._wndEngine = null;
        this._logger = null;
    },
    
    get controller() {
        return this._overlayController;
    },
    
    attachWidget: function GXB_attachWidget(aWidgetItem) {
        this._logger.debug("Attach widget. Widget item id: " + aWidgetItem.id);
        try {
            this._initWidget(aWidgetItem);
        }
        catch (e) {
            this._logger.error("Couldn't init widget. " + XB._base.formatException(e));
            this._logger.trace(e.stack);
        }
    },
    
    handleDataChange: function GXB_handleDataChange(widgetId, valUid, newValue) {
        let [uiElements, xbElement] = this._valWidgetMap.get(widgetId, valUid);
        this._logger.debug("Handle data change: " + [widgetId, valUid, newValue, uiElements ? uiElements.length : undefined, xbElement]);
        
        if (uiElements && uiElements.length && xbElement) {
            for (let elIndex = 0, len = uiElements.length; elIndex < len; elIndex++) {
                this._updateWidget(widgetId, uiElements[elIndex], xbElement, newValue);
            }
        }
    },
    
    showLazyWidget: function GXB_showLazyWidget(uiElement) {
        if (uiElement.__yaWgtShowing)
            return true;
        
        let [widgetId, xbElements] = this._valWidgetMap.getByUiElement(uiElement);
        this._logger.debug("Show lazy widget: " + [widgetId, uiElement.localName, xbElements]);
        
        let res = false;
        if (widgetId) {
            for (let i = 0, len = xbElements.length; i < len; i++) {
                res = this._constructWidgetDeeper(uiElement, widgetId, xbElements[i]) || res;
            }
        }
        
        return res;
    },
    
    hideLazyWidget: function GXB_hideLazyWidget(uiElement) {
        if (!uiElement.__yaWgtShowing)
            return true;
        
        this._logger.debug("Hide lazy widget: " + uiElement.localName);
        
        uiElement.__yaWgtShowing = false;
        
        while (uiElement.firstChild) {
            this._valWidgetMap.removeUiElement(uiElement.firstChild);
            uiElement.removeChild(uiElement.firstChild);
        }
        
        return true;
    },
    
    registerExtActionsHelper: function GXB_registerExtActionsHelper(handler) {
        if ( handler && (typeof handler.handleXBAction == "function") )
            this._extActionHandler = handler;
        else
            throw new TypeError("Provider helper does not support needed interface");
    },
    
    unregisterExtActionsHelper: function GXB_registerExtActionsHelper(handler) {
        if (handler && this._extActionHandler === handler)
            this._extActionHandler = null;
    },
    
    _presetDoc: null,
    _logger: XB._noopLogger,
    _valWidgetMap: null,
    _wndEngine: null,
    _extActionHandler: null,
    _fullWidgetItemIDPattern: null,
    _overlayController: null,
    
    _initWidget: function GXB__initWidget(aWidgetItem, aWidgetSettings) {
        this._logger.debug("Init widget. Widget item id: " + aWidgetItem.id);
        
        var widgetProtoId, widgeInstId, widgetSettings = aWidgetSettings;
        
        var fullMatch = aWidgetItem.id.match(this._fullWidgetItemIDPattern);
        if (fullMatch) {
            widgetProtoId = fullMatch[1];
            widgeInstId = fullMatch[2];
            
            if (widgeInstId == "0") {
                XB._base.application._overlayProvider._cloneItemToPalette(aWidgetItem, widgetProtoId);
                
                widgeInstId = XB._base.application.getNewWidgetInstanceID();
                aWidgetItem.id = aWidgetItem.id.replace(/0$/, widgeInstId);
            }
            
            var settingAttrPattern = /^xbsetting\-(.+)$/;
            var attrsMap = aWidgetItem.attributes;
            for (let len = attrsMap.length, idx = 0; idx < len; idx++ ) {
                var attrNode = attrsMap.item(idx);
                var match = attrNode.localName.match(settingAttrPattern);
                if (match) {
                    if (widgetSettings === undefined)
                        widgetSettings = {};
                    
                    var settingName = match[1];
                    var settingValue = attrNode.nodeValue;
                    
                    widgetSettings[settingName] = settingValue;
                }
            }
        }
        else {
            var presetIDPattern = new RegExp("^" + XB._base.application.name + "\\.cb\\-default\\-(\\d+)$");
            var presetIDMatch = aWidgetItem.id.match(presetIDPattern);
            if (presetIDMatch) {
                var presetItemIndex = presetIDMatch[1];
                let defaultIDsHash = XB._base.application.defaultIDsHash;
                
                if (aWidgetItem.parentNode && aWidgetItem.parentNode.localName == "toolbarpaletteitem")
                    delete defaultIDsHash[presetItemIndex];
                
                let createdID = defaultIDsHash[presetItemIndex] || null;
                
                var presetItemInfo = XB._base.application.defaultWidgetsInfo[presetItemIndex];
                if (!presetItemInfo)
                    throw new Error("No preset widget info " + presetItemIndex);
                
                widgetProtoId = presetItemInfo.protoID;
                widgeInstId = XB._base.application.getNewWidgetInstanceID();
                
                XB._base.application._overlayProvider._cloneItemToPalette(aWidgetItem, widgetProtoId);
                
                aWidgetItem.id = createdID || (XB._base.application.name + ".cb-" + widgetProtoId + "-inst-" + widgeInstId);
                defaultIDsHash[presetItemIndex] = aWidgetItem.id;
                
                widgetSettings = presetItemInfo.settings;
                
                if (createdID)
                    return this._initWidget(aWidgetItem, widgetSettings);
            }
            else
                throw new Error("Unrecognized ID pattern");
        }
        
        let widget = this._wndEngine.createWidget(widgetProtoId, widgeInstId, widgetSettings);
        let widgetProto = widget.getPrototype();
        let xbElement = widgetProto.getSrcElement();
        if (!xbElement) {
            this._logger.error("Can't find xbElement for widget (widget id: " + widgetId + ")");
            return;
        }
        
        this._constructWidget(aWidgetItem, widgeInstId, xbElement);
    },
    
    _constructWidget: function GXB__constructWidget(uiParentElement, widgetId, xbElement) {
        this._logger.debug("Construct widget: " + [uiParentElement.localName, xbElement.localName, xbElement.childNodes.length]);
        
        let doc = uiParentElement.ownerDocument;
        
        let _uiElement = null;
        let uiElement = null;
        let xbElements = null;
        
        let isValRef = {
            valUid: xbElement.getUserData(XB._base.consts.STR_VAL_REF_ID_KEY_NAME)
        };
        
        switch (xbElement.localName) {
            case "widget":
                break;
            
            case "button":
                uiElement = doc.createElementNS(XB._base.consts.STR_XUL_NS, "toolbarbutton");
                break;
            
            case "tooltip":
                uiElement = doc.createElementNS(XB._base.consts.STR_XUL_NS, "tooltip");
                
                let tooltipId = XB._base.application.name + ".cb-tooltip-" + (new Date().getTime());
                uiElement.setAttribute("id", tooltipId);
                uiParentElement.setAttribute("tooltip", tooltipId);
                break;
                
            case "menu":
                let menuPopup = doc.createElementNS(XB._base.consts.STR_XUL_NS, "menupopup");
                _uiElement = menuPopup;
                
                if (uiParentElement.localName == "menuitem") {
                    return;
                } else if (uiParentElement.localName == "menupopup") {
                    uiElement = doc.createElementNS(XB._base.consts.STR_XUL_NS, "menu");
                    uiElement.appendChild(menuPopup);
                } else {
                    uiElement = menuPopup;
                    uiParentElement.appendChild(menuPopup);
                }
                break;
            
            case "menuitem":
                uiElement = doc.createElementNS(XB._base.consts.STR_XUL_NS, "menuitem");
                break;
            
            case "grid":
                uiElement = doc.createElementNS(XB._base.consts.STR_HTML_NS, "table");
                break;
            
            case "row":
                uiElement = doc.createElementNS(XB._base.consts.STR_HTML_NS, "tr");
                break;
            
            case "cell":
                uiElement = doc.createElementNS(XB._base.consts.STR_HTML_NS, "td");
                var xbAttr = xbElement.attributes;
                for (let i = 0, len = xbAttr.length; i < len; i++) {
                    let attr = xbAttr.item(i);
                    let cssName = this.CSS_ATTR_NAMES[attr.name];
                    
                    if (cssName) {
                        uiElement.style[cssName] = attr.value;
                    } else if (attr.name == "merge-left") {
                        let td = uiParentElement.lastChild;
                        if (td && td.localName == "td") {
                            let colspan = parseInt(td.getAttribute("colspan"), 10) || 1;
                            td.setAttribute("colspan", colspan + 1);
                        }
                        return;
                    }
                    else
                        this._logger.debug("_constructWidget: can't handle style attribute '" + attr.name + "'");
                }
                break;
            
            case "separator":
                switch (uiParentElement.localName) {
                    case "menupopup":
                        uiElement = doc.createElementNS(XB._base.consts.STR_XUL_NS, "menuseparator");
                        break;
                    
                    case "toolbaritem":
                        uiElement = doc.createElementNS(XB._base.consts.STR_XUL_NS, "toolbarseparator");
                        break;
                }
                break;
            
            case "action":
                let actionCommand = xbElement.getAttribute("command") || "";
                for (let subIndex = 0, len = xbElement.childNodes.length; subIndex < len; subIndex++) {
                    let child = xbElement.childNodes[subIndex];
                    
                    if (child.nodeType === child.ELEMENT_NODE && child.localName === XB._base.consts.STR_VAL_REF_ELEM_NAME) {
                        actionCommand += (child.getUserData(XB._base.consts.STR_VAL_REF_ID_KEY_NAME) + ";");
                    }
                }
                
                actionCommand = widgetId + ":" + actionCommand;
                
                switch (uiParentElement.localName) {
                    case "toolbarbutton":
                    case "button":
                    case "menuitem":
                        if (actionCommand)
                            uiParentElement.setAttribute("barAction", actionCommand);
                        else if (uiParentElement.hasAttribute("barAction"))
                            uiParentElement.removeAttribute("barAction");
                        break;
                    
                    default:
                        break;
                }
                return;
                
            case "text":
                this._constructText(widgetId, uiParentElement, xbElement, isValRef);
                return;
                
            case "icon":
                let imageRelPath = this._extractStringValueFrom(widgetId, xbElement, isValRef);
                let imagePath = this._convertImageSpec(imageRelPath, widgetId);
                
                uiParentElement.setAttribute("image", imagePath);
                
                switch (uiParentElement.localName) {
                    case "menuitem":
                        uiParentElement.setAttribute("class", "menuitem-iconic");
                        break;
                    case "menu":
                        uiParentElement.setAttribute("class", "menu-iconic");
                        break;
                    default:
                        break;
                }
                
                break;
            
            case "enabled":
                break;
            
            case "checked":
                uiParentElement.setAttribute("checked", this._extractStringValueFrom(widgetId, xbElement, isValRef));
                switch (uiParentElement.localName) {
                    case "menuitem":
                        uiParentElement.setAttribute("type", "checkbox");
                        break;
                    default:
                        break;
                }
                break;
            
            case "__value__":
                var valUid = xbElement.getUserData(XB._base.consts.STR_VAL_REF_ID_KEY_NAME);
                isValRef.valUid = valUid;
                
                var value = this._wndEngine.getValue(widgetId, valUid);
                if (XB._base.runtime.isXML(value)) {
                    xbElements = value;
                    this._logger.debug("__value__: xbElements.length = " + xbElements.length + ", " + xbElements.item(0).localName);
                } else {
                    this._logger.debug("Can't handle element");
                }
                
                if (!xbElements)
                    return;
                
                break;
            
            default:
                this._logger.debug("constructWidget: unknown element '" + xbElement.localName + "'");
                return;
        }
        
        this._valWidgetMap.put(isValRef.valUid, widgetId, _uiElement || uiElement || uiParentElement, xbElement);
        
        if (uiElement) {
            let parentNode = uiParentElement;
            switch (xbElement.localName) {
                case "tooltip":
                    if (uiParentElement.localName == "toolbarbutton")
                        parentNode = uiParentElement.parentNode;
                    break;
                
                default:
                    break;
            }
            
            if (parentNode) {
                parentNode.appendChild(uiElement);
                
                switch (xbElement.localName) {
                    case "menu":
                        let parentType = "menu";
                        
                        if (parentNode.localName == "toolbarbutton")
                            parentType = "menu-button";
                        
                        parentNode.setAttribute("type", parentType);
                        break;

                    default:
                        break;
                }
            }
        }
        
        let includes,
            excludes;
        
        switch (xbElement.localName) {
            case "tooltip":
                return;
                
            case "menu":
                includes = ["text", "icon"];
                break;
            
            default:
                break;
        }
        
        xbElements = (xbElements || xbElement).childNodes;
        
        for (let propIndex = 0, len = xbElements.length; propIndex < len; propIndex++) {
            var propElement = xbElements[propIndex];
            if (propElement.nodeType != propElement.ELEMENT_NODE ||
                propElement.namespaceURI != XB._base.consts.STR_UI_NS)
                continue;
            
            if (includes && includes.indexOf(propElement.localName) == -1)
                continue;
            
            if (excludes && excludes.indexOf(propElement.localName) != -1)
                continue;
            
            this._constructWidget(uiElement || uiParentElement, widgetId, propElement);
        }
    },
    
    _constructWidgetDeeper: function GXB__constructWidgetDeeper(uiElement, widgetId, xbElement) {
        this._logger.debug("Construct widget deeper: " + [uiElement.localName, xbElement.localName]);
        
        switch (xbElement.localName) {
            case "tooltip":
            case "menu":
                break;
            
            default:
                return false;
        }
        
        uiElement.__yaWgtShowing = true;
        
        for (let propIndex = 0, len = xbElement.childNodes.length; propIndex < len; propIndex++) {
            var propElement = xbElement.childNodes[propIndex];
            if (propElement.nodeType != propElement.ELEMENT_NODE ||
                propElement.namespaceURI != XB._base.consts.STR_UI_NS)
                continue;
            
            this._constructWidget(uiElement, widgetId, propElement);
        }
        
        return true;
    },
    
    _updateWidget: function GXB__updateWidget(widgetId, uiElement, xbElement, newValue) {
        this._logger.debug("Update widget: " + [widgetId, uiElement.localName, xbElement.localName, newValue]);
        
        if (XB._base.runtime.isXML(newValue)) {
            if (uiElement.localName == "span" && uiElement.parentNode && uiElement.parentNode.childNodes.length < 4)
                this._constructRichText(widgetId, uiElement, newValue, {});
            
            return;
        }
        
        let xbType = xbElement.localName;
        if (xbType == "__value__") {
            switch (xbElement.parentNode.localName) {
                case "text":
                case "style":
                    xbType = "text";
                    break;
                
                default:
                    break;
            }
        }
        
        switch (xbType) {
            case "text":
                if ("yaTextContent" in uiElement) {
                    uiElement.yaTextContent = newValue;
                } else if ("textContent" in uiElement) {
                    uiElement.textContent = newValue;
                } else {
                    uiElement.setAttribute("label", newValue);
                }
                break;
            
            case "icon":
                uiElement.setAttribute("image", this._convertImageSpec(newValue, widgetId));
                break;
            
            case "attribute":
                var cssName = this.CSS_ATTR_NAMES[xbElement.getAttribute("name")];
                if (cssName)
                    uiElement.style[cssName] = this._extractStringValueFrom(widgetId, xbElement, {});
                else
                    this._logger.trace("_updateWidget: can't handle style attribute '" + xbElement.getAttribute("name") + "'");
                break;
            
            default:
                this._logger.trace("_updateWidget: unknown element '" + xbElement.localName + "'");
                break;
        }
    },
    
    _constructText: function GXB__constructText(widgetId, uiParentElement, xbElement, isValRef) {
        this._logger.debug("Construct text: " + [uiParentElement.localName, xbElement.localName]);
        
        if ("yaTextContent" in uiParentElement) {
            uiParentElement.yaTextContent = this._extractStringValueFrom(widgetId, xbElement, isValRef);
        } else if (uiParentElement.localName == "toolbarbutton" || uiParentElement.localName == "span") {
            if ("yaMenuToolbarButton" in uiParentElement)
                uiParentElement = uiParentElement.yaMenuToolbarButton;
            
            this._constructRichText(widgetId, uiParentElement, xbElement, isValRef);
            return;
            
        } else if (uiParentElement.namespaceURI == XB._base.consts.STR_HTML_NS) {
            this._constructRichText(widgetId, uiParentElement, xbElement, isValRef);
        } else {
            uiParentElement.setAttribute("label", this._extractStringValueFrom(widgetId, xbElement, isValRef));
        }
        
        this._valWidgetMap.put(isValRef.valUid, widgetId, uiParentElement, xbElement);
    },
    
    _constructRichText: function GXB__constructRichText(widgetId, uiParentElement, xbElement, isValRef) {
        let uiElement = null;
        let doc = uiParentElement.ownerDocument;
        
        this._logger.debug("Construct rich text: " + [xbElement.localName, xbElement.childNodes.length]);
        
        for (let subIndex = 0, len = xbElement.childNodes.length; subIndex < len; subIndex++) {
            var subNode = xbElement.childNodes[subIndex];
            this._logger.trace("  (sub)Construct rich text: " + [subNode.localName, subNode.nodeValue]);
            
            switch (subNode.nodeType) {
                case subNode.TEXT_NODE:
                    var val = sysutils.trimString(subNode.nodeValue);
                    if (val) {
                        this._logger.trace("found text node value " + val);
                        if (uiParentElement.localName == "toolbarbutton") {
                            uiElement = doc.createElementNS(XB._base.consts.STR_HTML_NS, "span");
                            uiElement.textContent = val;
                            uiParentElement.appendChild(uiElement);
                        } else if (uiParentElement.localName == "span") {
                            uiElement = doc.createElementNS(XB._base.consts.STR_HTML_NS, "span");
                            uiElement.textContent = val;
                            uiParentElement.appendChild(uiElement);
                        } else {
                            uiParentElement.textContent = val;
                        }
                    }
                    break;

                case subNode.ELEMENT_NODE:
                    if (subNode.namespaceURI == XB._base.consts.STR_UI_NS) {
                        switch (subNode.localName) {
                            case "style":
                                switch (uiParentElement.localName) {
                                    case "toolbarbutton":
                                    case "td":
                                    case "span":
                                        uiElement = doc.createElementNS(XB._base.consts.STR_HTML_NS, "span");
                                        uiParentElement.appendChild(uiElement);
                                        break;
                                    default:
                                        break;
                                }
                                
                                this._constructRichText(widgetId, uiElement, subNode, isValRef);
                                
                                let styleAttr = subNode.attributes;
                                for (let i = 0, len = styleAttr.length; i < len; i++) {
                                    let attr = styleAttr.item(i);
                                    let cssName = this.CSS_ATTR_NAMES[attr.name];
                                    
                                    if (cssName)
                                        uiElement.style[cssName] = attr.value;
                                    else
                                        this._logger.trace("  (sub)Construct rich text: can't handle style attribute '" + attr.name + "'");
                                }
                                
                                this._valWidgetMap.put(isValRef.valUid, widgetId, uiElement, subNode);
                                break;
                            
                            case "attribute":
                                let styleAttrName = subNode.getAttribute("name");
                                
                                let cssName = this.CSS_ATTR_NAMES[styleAttrName];
                                uiParentElement.style[cssName] = this._extractStringValueFrom(widgetId, subNode, isValRef);
                                
                                this._valWidgetMap.put(isValRef.valUid, widgetId, uiParentElement, subNode);
                                
                                break;
                            
                            case XB._base.consts.STR_VAL_REF_ELEM_NAME:
                                switch (uiParentElement.localName) {
                                    case "toolbarbutton":
                                    case "td":
                                    case "span":
                                        uiElement = doc.createElementNS(XB._base.consts.STR_HTML_NS, "span");
                                        uiParentElement.appendChild(uiElement);
                                        break;
                                    default:
                                        break;
                                }
                                
                                var valUid = subNode.getUserData(XB._base.consts.STR_VAL_REF_ID_KEY_NAME);
                                if (!valUid)
                                    this._logger.warn("  (sub)Construct rich text: no valUid for subNode");
                                
                                var value = this._wndEngine.getValue(widgetId, valUid);
                                this._valWidgetMap.put(valUid, widgetId, uiElement, subNode);
                                
                                if (XB._base.runtime.isXML(value)) {
                                    this._constructRichText(widgetId, uiParentElement, value, isValRef);
                                } else {
                                    uiElement.textContent = value;
                                }
                                break;
                            
                            default:
                                this._logger.trace("  (sub)Construct rich text: can't handle node. [" + subNode.localName + "]");
                                break;
                        }
                        
                    } else {
                        this._logger.trace("  (sub)Construct rich text: can't handle node. [" + subNode.localName + "]");
                    }
                    break;
                
                default:
                    this._logger.trace("  (sub)Construct rich text: can't handle node. [" + subNode.localName + "]");
                    break;
            }
        }
    },
    
    _extractStringValueFrom: function GXB__extractStringValueFrom(widgetId, xbElement, isValRef) {
        var result = "";
        
        this._logger.trace("Extract string value from: " + [widgetId, xbElement, isValRef]);
        
        for (let subIndex = 0, len = xbElement.childNodes.length; subIndex < len; subIndex++) {
            var subNode = xbElement.childNodes[subIndex];
            switch (subNode.nodeType) {
                case subNode.TEXT_NODE:
                    var val = sysutils.trimString(subNode.nodeValue);
                    if (val) {
                        this._logger.trace("found text node value " + val);
                        result += val;
                    }
                    break;
                
                case subNode.ELEMENT_NODE:
                    if (subNode.namespaceURI == XB._base.consts.STR_UI_NS && 
                        subNode.localName == XB._base.consts.STR_VAL_REF_ELEM_NAME) {
                        
                        var valUid = subNode.getUserData(XB._base.consts.STR_VAL_REF_ID_KEY_NAME);
                        isValRef.flag = true;
                        isValRef.valUid = valUid;
                        
                        result += this._wndEngine.getValueAsString(widgetId, valUid);
                    }
                    break;
                
                default:
                    this._logger.trace("Extract string value from: can't handle node. [" + subNode.localName + "]");
                    break;
            }
        }
        
        this._logger.trace("Resulting text is '" + result + "'");
        
        return result;
    },
    
    _convertImageSpec: function GXB__convertImageSpec(aURL, widgetId) {
        if (!aURL)
            return "";
        
        if (!(/^http(s)?:\/\//i.test(aURL) || aURL.indexOf("data:image/") == 0)) {
            aURL = this._wndEngine.getWidget(widgetId).getPrototype().getLibrary().libPackage.getXBURL(aURL);
        }
        
        let uri = sysutils.createFixupURIFromString(aURL);
        return uri ? uri.spec : "";
    },
    
    onCommand: function GXB_onCommand(aEvent) {
        let target = aEvent.target;
        let barAction = target.hasAttribute("barAction") ? target.getAttribute("barAction") : "";
        
        let widgetId, actions;
        
        let separatorIndex = barAction.indexOf(":");
        if (separatorIndex > 0) {
            widgetId = barAction.substr(0, separatorIndex);
            actions = barAction.substr(separatorIndex + 1).split(";");
        }
        
        this._logger.debug("onCommand" +
                           "\n Target: " + target.localName +
                           "\n Action value: " + actions.join(",") +
                           "\n WidgetId value: " + widgetId);
        
        return this._actionsHelper(aEvent, widgetId, actions);
    },
    
    _actionsHelper: function GXB__actionsHelper(aEvent, aWidgetId, aActions) {
        if (!(aWidgetId && aActions))
            return false;
        
        let stopPropagation = true;
        
        for each (let aAction in aActions) {
            if (!aAction) continue;
            
            try {
                this._wndEngine.perform(aWidgetId, aAction);
            }
            catch (e) {
                try {
                    if (e instanceof XB.WindowEngine.ENoActionHandler) {
                        if (!this._extActionHandler)
                            throw new XB.WindowEngine.ENoActionHandler("Extened actions handler is not registered");
                        this._extActionHandler.handleXBAction(aWidgetId, aAction);
                    }
                    else
                        throw e;
                }
                catch (e) {
                    this._logger.error("Error running action " +
                                       [aWidgetId, aAction].join(":") + ". " +
                                       XB._base.formatException(e) );
                    this._logger.debug(e.stack);
                }
            }
        }
        if (stopPropagation) {
            aEvent.stopPropagation();
            aEvent.preventDefault();
        }
        
        return true;
    }
};