const CS = Components.classes["@mozilla.org/consoleservice;1"].getService(Components.interfaces.nsIConsoleService);
const XULNS = "http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul";

var ysXBQuoteSetupDLgController = {
    onLoad: function xbqs_onLoad() {
        document.documentElement.getButton("accept").disabled = true;
        
        try {
            this._retVal = window.arguments[0];
            this._quoteTypesList = document.getElementById("quoteTypesList");
            this._quoteNamesList = document.getElementById("quoteNamesList");
            this._quoteTypesList.addEventListener("select", this, false);
            
            this._quotesListDataNode = window.arguments[1];
            var quotesListXML = this._quotesListDataNode.getValue(this);
            if (quotesListXML) {
                this._updateList(quotesListXML);
            }
            else {
                this._timeoutTimer = Components.classes["@mozilla.org/timer;1"]
                    .createInstance(Components.interfaces.nsITimer);
                this._timeoutTimer.initWithCallback(this, 90000, this._timeoutTimer.TYPE_ONE_SHOT);
            }
        }
        catch (e) {
            CS.logStringMessage(e.message + "\n" + e.stack);
        }
    },
    
    onUnload: function xbqs_onUnload() {
        this._finish();
    },
    
    onCancel: function xbqs_onCancel() {
        this._retVal.value = undefined;
        return true;
    },
    
    onAccept: function xbqs_onAccept() {
        this._retVal.value = this._quoteNamesList.value;
        return true;
    },
    
    handleEvent: function xbqs_handleEvent(event) {
        if (event.target != this._quoteTypesList)
            return;
        
        switch (event.type) {
            case "select":
                this._onQuoteGroupChanged();
                break;
        }
    },
    
    notify: function xbqs_notify(timer) {
        if (timer != this._timeoutTimer) return;
        this._quoteTypesList.selectedIndex = 1;
    },
    
    getEffectiveID: function xbqs_getEffectiveID() {
        return "ysXBQuoteSetupDLgController";
    },
    
    _getHumanReadableID: function xbqs_getHumanReadableID() {
        return this.getEffectiveID();
    },
    
    freeze: function xbqs_freeze() {
        
    },
    
    melt: function xbqs_melt(changedNode) {
        if (changedNode != this._quotesListDataNode) return;
        if (this._timeoutTimer)
            this._timeoutTimer.cancel();
        var quotesListXML = this._quotesListDataNode.getValue();
        if (quotesListXML) {
            this._updateList(quotesListXML);
        }
        else {
            this._quoteTypesList.selectedIndex = 1;
        }
    },
    
    _retVal: null,
    _quotesListDataNode: null,
    _quotesGroups: null,
    _quoteTypesList: null,
    _quoteNamesList: null,
    _timeoutTimer: null,
    
    _updateList: function xbqs_updateList(quotesListXML) {
        this._quoteTypesList.selectedIndex = -1;
        var listsSelectionInfo = this._fillQuotes(quotesListXML.item(0));
        this._quoteTypesList.selectedIndex = listsSelectionInfo.groupIndex;
        this._quoteNamesList.selectedIndex = listsSelectionInfo.quoteIndex;
    },
    
    _finish: function xbqs_finish() {
        this._quoteTypesList.removeEventListener("select", this, false);
        if (this._timeoutTimer)
            this._timeoutTimer.cancel();
        this._timeoutTimer = null;
        if (this._quotesListDataNode)
            this._quotesListDataNode.unsubscribe(this);
    },
    
    _enableAcceptButton: function xbqs_enableAcceptButton() {
        document.documentElement.getButton("accept").disabled = !!(this._quoteTypesList.disabled || this._quoteNamesList.disabled);
    },
    
    _enableMenulist: function xbqs_enableMenulist(aMenulist) {
        aMenulist.disabled = !aMenulist.menupopup.childNodes.length;
        this._enableAcceptButton();
    },
    
    _fillQuotes: function xbqs_fillQuotes(quotesNode) {
        this._quotesGroups = [];
        var currListSelectionInfo = {
            groupIndex: 0,
            quoteIndex: 0
        };
        var currQuoteID = this._retVal.value;
        var quoteTypesPopup = this._quoteTypesList.menupopup;
        
        var first = null;
        while ((first = quoteTypesPopup.firstChild)) {
            quoteTypesPopup.removeChild(first);
        }
        
        for each (let groupNode in quotesNode.childNodes) {
            if ( (groupNode.nodeType != groupNode.ELEMENT_NODE) || (groupNode.nodeName != "group") )
                continue;
            
            var groupName = groupNode.getAttribute("title");
            var groupIndex = this._quotesGroups.length;
            
            var groupMenuItem = document.createElementNS(XULNS, "menuitem");
            groupMenuItem.setAttribute("label", groupName);
            groupMenuItem.setAttribute("value", groupIndex);
            quoteTypesPopup.appendChild(groupMenuItem);
            
            var groupQuotes = [];
            for each (let quoteNode in groupNode.childNodes) {
                if ( (quoteNode.nodeType != quoteNode.ELEMENT_NODE) || (quoteNode.nodeName != "quote") )
                    continue;
                
                var quoteID = quoteNode.getAttribute("id");
                if (quoteID == currQuoteID) {
                    currListSelectionInfo.groupIndex = groupIndex;
                    currListSelectionInfo.quoteIndex = groupQuotes.length;
                }
                
                var quoteName = "";
                for each (let quoteSubNode in quoteNode.childNodes) {
                    if ((quoteSubNode.nodeType != quoteSubNode.ELEMENT_NODE) ||
                        (quoteSubNode.nodeName != "name")) continue;
                    quoteName = quoteSubNode.textContent;
                }
                
                var quote = {
                    id: quoteID,
                    name: quoteName
                };
                
                groupQuotes.push(quote);
            }
            
            this._quotesGroups.push(groupQuotes);
        }
        this._enableMenulist(this._quoteTypesList);
        
        return currListSelectionInfo;
    },
    
    _onQuoteGroupChanged: function xbqs_onQuoteGroupChanged() {
        let quoteNamesListPopup = this._quoteNamesList.menupopup;
        var first = null;
        while ((first = quoteNamesListPopup.firstChild)) {
            quoteNamesListPopup.removeChild(first);
        }
        
        var selectedItem = this._quoteTypesList.selectedItem;
        if (!selectedItem) return;
        var group = this._quotesGroups[ selectedItem.value ];
        for each (let quote in group) {
            var quoteMenuItem = document.createElementNS(XULNS, "menuitem");
            quoteMenuItem.setAttribute("label", quote.name);
            quoteMenuItem.setAttribute("value", quote.id);
            quoteNamesListPopup.appendChild(quoteMenuItem);
        }
        this._enableMenulist(this._quoteNamesList);
        this._quoteNamesList.selectedIndex = 0;
    }
}