var YaProgressListener =  (function() {
  /* ************************* *
   * tabHandler
   * ************************* */
  function tabHandler(aBrowser) {
    this._browser = aBrowser;
    this.actionValue = null;
    
    this.location = null;
    
    this.times = null;
    this.clearTimes();
    
    this.inProgress = false;
    this.isFromCache = false;
    this.data = {};
    
    this._tabData = {};//data for searchbox, highlighter, etc.
    
    this._setListeners(true);
  }
  
  tabHandler.prototype = {
    _events: [
      ["DOMContentLoaded", false],
      ["load", true],
      ["pageshow", true],
      ["pagehide", true]
    ],
    
    _setListeners: function(aAdd) {
      let fn = (aAdd ? "add" : "remove") + "EventListener";
      
      this._events.forEach(function(ev) {
        this._browser[fn](ev[0], this, ev[1], true);
      }, this);
    },
    
    VERSION: 3,
    
    get tabData() {
      return this._tabData;
    },
    
    getRequestName: function(aRequest) {
      let name = null;
      
      try {
        name = aRequest.name;
        name = /^https?/.test(name) ? name : null;
      } catch(e) {}
      
      return name;
    },
    
    handleEvent: function(aEvent) {
      if (!(aEvent.isTrusted && this._isTargetEqTopWindow(aEvent)))
        return;
      
      switch (aEvent.type) {
        case "DOMContentLoaded":
          this.times.documentLoaded = this.timeNow;
          
          YaProgressListener.notifyListeners("DOMContentLoaded", this._browser);
          
          break;
        
        case "load":
          this.times.windowLoaded = this.timeNow;
          
          this.onNavigationStopped();
          
          YaProgressListener.notifyListeners("load", this._browser);
          
          break;
        
        case "pagehide":
          YaProgressListener.notifyListeners("pagehide", this._browser);
          
          break;
        
        case "pageshow":
          YaProgressListener.notifyListeners("pageshow", this._browser);
          
          if (this._browser !== gBrowser.mCurrentBrowser)
            Ya.nsIYa.webProgressListener.onPageShowInBackground(this._browser.webProgress, Ya.buttonsObject);
          
          break;
      }
    },
    
    _isTargetEqTopWindow: function(aEvent) {
      let target = aEvent.originalTarget;
      
      if (!(target && (target instanceof Ci.nsIDOMHTMLDocument)))
        return false;
      
      let targetDefView = (new XPCNativeWrapper(target, "defaultView")).defaultView;
      let topDefView = (new XPCNativeWrapper(this._browser.contentDocument, "defaultView")).defaultView;
      
      return !!(targetDefView === topDefView);
    },
    
    destroy: function() {
      this._setListeners(false);
      
      this._browser = null;
      this.actionValue = null;
      this.times = null;
    },
    
    get timeNow() {
      return new Date().getTime();
    },
    
    clearTimes: function() {
      this.times = {
        transferStart:   -1,
        transferStop:    -1,
        documentLoaded:  -1,
        windowLoaded:    -1
      }
    },
    
    onTransferStart: function(aURL) {
      this.clearTimes();
      this.times.transferStart = this.timeNow;
      
      this.location = aURL || null;
      this.inProgress = true;
      this.isFromCache = false;
    },
    
    onTransferStop: function() {
      if (this.times.transferStart > 0)
        this.times.transferStop = this.timeNow;
    },
    
    onNavigationStopped: function() {
      this.inProgress = false;
      
      let data = {};
      for each (var prop in ["location", "isFromCache", "actionValue"])
        data[prop] = this[prop];
      
      let times = this.times;
      if (times.transferStart > 0 && times.transferStop > 0 &&
          times.documentLoaded > 0 && times.windowLoaded > 0) {
        
        let t1 = times.transferStop - times.transferStart;
        let t2 = times.documentLoaded - times.transferStop;
        let t3 = times.windowLoaded - times.documentLoaded;
        
        if (t1 > 0 && t2 > 0 && t3 >= 0)
          data.times = [t1,t2,t3];
      }
      
      this.data = data;
      this.actionValue = null;
      
      this.clearTimes();
    },
    
    hostQuickRe: /(^|\.)((yandex|ya|moikrug)\.(ru|com|ua|by|kz)|(google)\.(com|ru)|(mail|rambler)\.ru)$/i,
    hostYaRe: new RegExp("(^|\\.)(yandex\\.([A-Za-z]{2,4})|(ya|moikrug)\\.ru)$", "i"),
    hostOnlyRe: new RegExp("(^|www\\.)(yandex|ya|moikrug|google|mail|rambler)\\.", "i"),
    hostReS: new RegExp("(^|(nova|www|go)\\.)(yandex|google|mail|rambler)\\.(?:com|ru|ua|by|kz)$", "i"),
    pathReS: new RegExp("^((yand)?search|srch)\\?", "i"),
    hostReM: new RegExp("^((web)?mail\\.yandex\\.(ru|ua|by|kz)|win\\.mail\\.ru|mail\\.rambler\\.ru)$", "i"),
    urlReM:  new RegExp("^http:\/\/([^\/]+\/(cgi\\-bin\/sendmsg\\?)?compose|webmail\\.yandex\\.(ru|ua|by|kz)\/messages|mail\\.yandex\\.(ru|ua|by|kz)\/((classic|modern|neo)/)?(messages|compose)|win\\.mail\\.ru\/cgi\\-bin\/(sentmsg\\?compose|msglist\\?folder=0&))|mail\\.rambler\\.ru\/mail\/(startpage\\?|mail\.cgi\\?(r|mode=(startpage|compose|mailbox;mbox=INBOX)))", "i"),
    
    checkConditions: function(aURL) {
      if (aURL && aURL.match(/^http:\/\/([^\/]+)\/?(.*)/)) {
        let host = RegExp.$1,
            path = RegExp.$2;
        
        if (this.hostQuickRe.test(host)) {
          if (!path && this.hostOnlyRe.test(host))
            return true;
          
          if (this.hostReM.test(host) && aURL.match(this.urlReM))
            return true;
          
          if (this.pathReS.test(path) && host.match(this.hostReS)) {
            let q = "text";
            switch (RegExp.$3) {
              case "google":  q = "(as_)?q";         break;
              case "mail":    q = "q";               break;
              case "rambler": q = "(query|words)";   break;
            }
            
            let re = new RegExp("[\?&](?:" + q + ")=([^#&\?]*)");
            if (re.test(path))
              return true;
          }
          
          if (this.hostYaRe.test(host))
            return true;
        }
      }
      
      return false;
    },
    
    isYandexHost: function(aURL) {
      if (aURL && aURL.match(/^http:\/\/([^\/]+)\/?(.*)/)) {
        if (this.hostYaRe.test(RegExp.$1))
          return true;
      }
      
      return false;
    },
    
    get dataSumm() {
      let res = {};
      
      if (this._browser && !this.inProgress) {
        let d = this.data;
        
        if (!d.isFromCache && d.times && d.location && 
            this._browser.currentURI.spec == d.location &&
            this.checkConditions(d.location)) {
          
          let t = [];
          d.times.forEach(function(aTime) {
            t.push(parseInt(aTime/10, 10));
          });
          
          let yamm = null;
          
          if (this.isYandexHost(d.location)) {
            let doc = this._browser.contentDocument;
            let metaNodes = doc.getElementsByTagName("meta");
            for (let i = 0, len = metaNodes.length; i < len; i++) {
              let metaName = (metaNodes[i].name || "").toLowerCase();
              if (metaName == "yamm" && metaNodes[i].content) {
                yamm = ("" + metaNodes[i].content).substr(0,10);
                break;
              }
            }
          }
          
          res.time = {
            tv: this.VERSION,
            t: t.join("-"),
            yamm: yamm
          };
        }
        
        res.action = d.actionValue;
        
        this.data = {};
      }
      
      return res;
    }
  };
  
  /* ************************* *
   * winProgressListener
   * ************************* */
  var winProgressListener = {
    QueryInterface: function(aIID) {
      if (aIID.equals(Ci.nsIWebProgressListener) ||
          aIID.equals(Ci.nsISupportsWeakReference) ||
          aIID.equals(Ci.nsISupports))
        return this;
      
      throw Components.results.NS_NOINTERFACE;
    },
    
    get nsIYaListener() {
      delete this.nsIYaListener;
      return this.nsIYaListener = Cc["@yandex.ru/yasearch;1"].getService(Ci.nsIYaSearch)
                                                             .wrappedJSObject
                                                             .webProgressListener;
    },
    
    onLocationChange: function(aWebProgress, aRequest, aLocation) {
      let top;
      try {
        let reqWindow = aWebProgress.DOMWindow;
        top = reqWindow === reqWindow.top;
      } catch(e) {}
      
      if (!top)
        return;
      
      aWebProgress.QueryInterface(Ci.nsIWebNavigation);
      aWebProgress.QueryInterface(Ci.nsIDocShell);
      
      YaProgressListener.notifyListeners("onWindowLocationChange", aWebProgress, aRequest, aLocation);
      
      this.nsIYaListener.onLocationChange(aWebProgress, Ya.buttonsObject);
    },
    
    onStateChange: function(aWebProgress, aRequest, aStateFlags, aStatus) {
      if (!aRequest || !(aStateFlags & Ci.nsIWebProgressListener.STATE_IS_NETWORK))
        return;
      
      aWebProgress.QueryInterface(Ci.nsIWebNavigation);
      aWebProgress.QueryInterface(Ci.nsIDocShell);
      
      if (aStateFlags & Ci.nsIWebProgressListener.STATE_START)
        this.nsIYaListener.onPageStateStart(aWebProgress, aRequest);
      
      if (aStateFlags & Ci.nsIWebProgressListener.STATE_STOP)
        this.nsIYaListener.onPageStateStop(aWebProgress, aRequest);
    },
    
    onProgressChange:    function(a,b,c,d,e,f){},
    onStatusChange:      function(a,b,c,d){},
    onSecurityChange:    function(a,b,c){},
    onLinkIconAvailable: function(a){}
  };
  
  /* ************************* *
   * YaProgressListener
   * ************************* */
  return {
    handleEvent: function(aEvent) {
      let browser,
          eventType = aEvent.type;
      
      switch (eventType) {
        case "TabOpen":
        case "TabClose":
          browser = aEvent.target.linkedBrowser;
          break;
      }
      
      if (!browser)
        return;
      
      switch (eventType) {
        case "TabOpen":
          this.addTabListener(browser);
          break;
        
        case "TabClose":
          this.removeTabListener(browser);
          break;
      }
    },
    
    init: function() {
      gBrowser.addProgressListener(winProgressListener, Ci.nsIWebProgress.NOTIFY_STATE_DOCUMENT);
      
      let panelsLen = gBrowser.mPanelContainer.childNodes.length;
      for (var i = 0; i < panelsLen; i++) {
        try {
          this.addTabListener(gBrowser.getBrowserAtIndex(i));
        } catch(e) {}
      }
      
      let container = gBrowser.tabContainer;
      container.addEventListener("TabOpen", this, false);
      container.addEventListener("TabClose", this, false);
    },
    
    destroy: function() {
      let panelsLen = gBrowser.mPanelContainer.childNodes.length;
      for (var i = 0; i < panelsLen; i++) {
        try {
          this.removeTabListener(gBrowser.getBrowserAtIndex(i));
        } catch(e) {}
      }
      
      let container = gBrowser.tabContainer;
      container.removeEventListener("TabOpen", this, false);
      container.removeEventListener("TabClose", this, false);
      
      gBrowser.removeProgressListener(winProgressListener);
    },
    
    addTabListener: function(aBrowser) {
      if (!("yaSearchTHandler" in aBrowser)) {
        aBrowser.yaSearchTHandler = new tabHandler(aBrowser);
      }
    },
    
    removeTabListener: function(aBrowser) {
      if ("yaSearchTHandler" in aBrowser) {
        aBrowser.yaSearchTHandler.destroy();
        aBrowser.yaSearchTHandler = null;
      }
    },
    
    _listeners: [],
    
    addListener: function(aHandler) {
      if (!this._listeners.some(function(aListener) { return aListener === aHandler; }))
        this._listeners.push(aHandler);
    },
    
    removeListener: function(aHandler) {
      this._listeners = this._listeners.filter(function(aListener){ return aListener !== aHandler; });
    },
    
    notifyListeners: function() {
      var _args = arguments;
      
      this._listeners.forEach(function(aListener) {
        try {//bug#83635?
          aListener.observe.apply(aListener, _args);
        } catch(e) {
          Ya.log(e);
        }
      });
    }
  }
})();

window.addEventListener("load", function(aLoadEvent) {
  aLoadEvent.currentTarget.removeEventListener("load", arguments.callee, false);
  
  aLoadEvent.currentTarget.addEventListener("unload", function(aUnloadEvent) {
    aUnloadEvent.currentTarget.removeEventListener("unload", arguments.callee, false);
    YaProgressListener.destroy();
  }, false);
  
  YaProgressListener.init();
}, false);