var gYaCity = {
  get WEATHER_IMAGES_MAP() {
    delete this.WEATHER_IMAGES_MAP;
    this.WEATHER_IMAGES_MAP = {};
    
    var WIMAP = {
      "i-rain-summer": ["1","n1","4","n4"],
      "i-snow": ["2","n2"],
      "i-cloud": ["5","16"],
      "i-ch": ["6"],
      "i-sun": ["7"],
      "i-rain-ch-summer": ["9","14"],
      "i-rain-ch2-summer": ["10"],
      "i-cloud-rain": ["11","15"],
      "i-ch-snow": ["12"],
      "i-cloud-snow": ["13"],
      "i-night-cloud": ["n5","n6"],
      "i-night-cloud-rain": ["n9","n10","n11","n14","n15"],
      "i-night-cloud-snow": ["n12","n13"],
      "i-night": ["n7","n16"],
      "i-rain-grad": ["3","n3"],
      "i-thund-rain": ["8","n8"],
      "i-snowstorm": ["17","n17"],
      "i-fog": ["18","n18"]
    };
    
    for (var wType in WIMAP)
      for each (var wSrc in WIMAP[wType])
        this.WEATHER_IMAGES_MAP[wSrc] = wType;
    
    return this.WEATHER_IMAGES_MAP;
  },
  
  _timedHTTPRequester: null,
  
  _timedHTTPRequesterCallback: function(aResponseData) {
    var data = gYaSearchService.safeE4Xml(aResponseData, null, ["geobase"]);
    return (data && data..region.length() > 10) ? aResponseData : null;
  },
  
  _init: function() {
    OBSERVER_SERVICE.addObserver(this, "quit-application-granted", false);
    OBSERVER_SERVICE.addObserver(this, "profile-after-change", false);
    OBSERVER_SERVICE.addObserver(this, "profile-before-change", false);
  },
  
  _uninit: function() {
    OBSERVER_SERVICE.removeObserver(this, "profile-before-change");
    OBSERVER_SERVICE.removeObserver(this, "profile-after-change");
    OBSERVER_SERVICE.removeObserver(this, "quit-application-granted");
  },
  
  _profileInit: function() {
    this._timedHTTPRequester = new G_TimedHTTPRequester(
        "cities", "http://export.yandex.ru/bar/geobase.xml", this.__citiesDataFileName,
        this._timedHTTPRequesterCallback, 14, 3 * 60//2 weeks, 3 hour
    );
  },
  
  _profileUninit: function() {
    if (gYaInstaller && !gYaInstaller.isBarUninstalled)
      this.flushRegionsData();
    
    this.clearGeoData();
    this.__stringBundle = null;
  },
  
  observe: function(aSubject, aTopic, aData) {
    switch (aTopic) {
      case "quit-application":
        this._uninit();
        break;
      
      case "profile-after-change":
        this._profileInit();
        break;
      
      case "profile-before-change":
        this._profileUninit();
        break;
    }
  },
  
  _observers: {},
  
  get observersLength() {
    var i = 0;
    for each (var observer in this._observers)
      i++;
    
    return i;
  },
  
  newToolbaritem: function(aId, aDocument) {
    return gYaSearchService.DOMUtils.adoptNode(
                gYaSearchService.getDOMDocContent("city-item",
                    gYaSearchService.domParser.parseFromString("<root id='" + aId + "'/>", "text/xml")
                ), aDocument);
  },
  
  addNewToolbaritem: function(aBeforeToolbaritemId) {
    let [newItemId, newItemNode] = gYaOverlay.insertToolbarItem("yasearch-city", aBeforeToolbaritemId);
    return newItemNode ? newItemId : null;
  },
  
  removeToolbaritem: function(aId) {
    var props = this.getCityProps(aId);
    
    if (props.buttonId) {
      OBSERVER_SERVICE.notifyObservers(null, "Ya-City-Item-OnRemove", props.buttonId);
      
      var cityNodes = this._observers[props.buttonId].nodes;
      
      for (var i = cityNodes.length; i-- > 0;) {
        var cityNode = cityNodes[i];
        if (cityNode) {
          var toolbar = gYaSearchService._getToolbarForNode(cityNode);
          cityNode.parentNode.removeChild(cityNode);
          gYaSearchService._persistToolbarSet(toolbar);
          
          this.removeObserver(cityNode);
        }
      }
    }
  },
  
  openDialog: function(aWindow, aToolbarItemId, aRemoveButtonOnCancel) {
    aWindow.openDialog(CHROME_CONTENT + "city/city-settings.xul", "",
                       "centerscreen,chrome,dialog,resizable,dependent",
                       aToolbarItemId, aRemoveButtonOnCancel);
  },
  
  openCityAliasDialog: function(aWindow, aToolbarItemId) {
    var props = this.getCityProps(aToolbarItemId);
    
    var buttonLabel = props.buttonLabel.toString();
    
    var aliasNameObject = { value: buttonLabel };
    
    var rv = gYaSearchService.promptService.prompt(aWindow,
                    this.getString("CityAliasTitle"),
                    this.getString("CityAliasText") + ":",
                    aliasNameObject, null, {});
    
    if (rv && aliasNameObject.value != "" && aliasNameObject.value != buttonLabel)
      this.updateButtonProps(aToolbarItemId, {labelAlias: aliasNameObject.value});
  },
  
  getURLForRegionId: function(aToolbarItemId, aTypeOfURL) {
    var res = null;
    
    if (aTypeOfURL != "weather" && aTypeOfURL != "traffic")
      return res;
    
    var props = this.getCityProps(aToolbarItemId);
    var region = this.regionsData.regions.region.(@id == props.cityId)[0];
    
    if (region) {
      var prefix = aTypeOfURL[0];
      var url = region["@" + prefix + "url"].toString();
      //~ var timestamp = parseInt(region['@' + prefix + 'ts'].toString(), 10);// - ... > G_TIME_NOW
      
      if (url && /^https?:\/\//.test(url))
        res = url;
    }
    
    if (!res && !props.cityId) {
      switch (aTypeOfURL) {
        case "weather":
          res = "http://weather.yandex.ru";
          break;
        
        case "traffic":
          res = "http://maps.yandex.ru";
          break;
      }
    }
    
    return res;
  },
  
  canRequestDataForCity: function(aCityProps) {
    if (aCityProps.weatherEnabled)
      return true;
    
    if (!aCityProps.trafficEnabled)
      return false;
    
    var region = this.regionsData.regions.region.(@id == aCityProps.cityId)[0];
    if (region && region.@tid.toString() == "null") {
      var nextCheckTime = parseInt(region.@tts.toString(), 10) + 60 * MIN_SEC;
      if (nextCheckTime > G_TIME_NOW)
        return false;
    }
    
    return true;
  },
  
  fillTooltip: function(aToolbarItemId, aTooltipContainer, aTooltipType) {
    var xmlDataStr = this.getDataForCityId(aToolbarItemId).xmlData.toString();
    
    var tooltipContent = gYaSearchService.getDOMDocContent("city-tooltip-" + aTooltipType, gYaSearchService.domParser.parseFromString(xmlDataStr, "text/xml"));
    
    aTooltipContainer.firstChild ?
        gYaSearchService.DOMUtils.replaceNode(tooltipContent, aTooltipContainer.firstChild) :
        gYaSearchService.DOMUtils.appendNode(tooltipContent, aTooltipContainer);
  },
  
  switchButtonProp: function(aToolbarItemId, aType) {
    var data = this.getDataForCityId(aToolbarItemId);
    
    switch (aType) {
      case "weather":
        this.updateButtonProps(aToolbarItemId, {weatherEnabledValue: data.weatherEnabled ? "0" : "1"});
        break;
      
      case "traffic":
        this.updateButtonProps(aToolbarItemId, {trafficEnabledValue: data.trafficEnabled ? "0" : "1"});
        break;
    }
  },
  
  _setData: function(aCityId, aData, aIsDataFromRequest) {
    for each (var observer in this._observers) {
      var props = this.getCityProps(observer.nodes[0].id);
      if (props.cityId === aCityId)
        this.__setData(props.buttonId, aData, aCityId, aIsDataFromRequest);
    }
  },
  
  __setData: function(aButtonId, aData, aCityId, aIsDataFromRequest) {
    var data = gYaSearchService.safeE4Xml(aData, "<info/>", ["info"]);
    
    // validation
    var dayPart = data.weather.day.day_part;
    var imgStr = dayPart.image.toString();
    
    if (!/^chrome:\/\//.test(imgStr)) {//not yet, fresh data
      var img = imgStr.match(/http\:\/\/weather\.yandex\.ru\/i\/(.+)\.png/) ? RegExp.$1 : "";
      var imgPngName = this.WEATHER_IMAGES_MAP[img];
      
      dayPart.image = CHROME_IMAGES + "weather" + (imgPngName ? ("/" + imgPngName) : "") + ".png";
    }
    
    if (aData && aIsDataFromRequest) {
      var regions = this.regionsData.regions;
      var region = regions.region.(@id == aCityId)[0];
      
      if (!region) {
        region = <region/>;
        region.@id = aCityId;
      }
      
      var weatherRegionId = data.weather.@region.toString();
      var weatherURL = data.weather.url.toString();
      
      if (weatherRegionId && /http:\/\//.test(weatherURL)) {
        region.@wid = weatherRegionId;
        region.@wurl = weatherURL;
        region.@wts = G_TIME_NOW;
      }
      
      var trafficRegionId = data.traffic.@region.toString();
      var trafficURL = data.traffic.url.toString();
      
      if (trafficRegionId && /http:\/\//.test(trafficURL)) {
        region.@tid = trafficRegionId;
        region.@turl = trafficURL;
        region.@tts = G_TIME_NOW;
        
        var length = +(data.traffic.length.toString());
        if (length >= 1)
          data.traffic.length = (length/1000).toString().replace(/\.(\d?)\d*/, ",$1");
        else
          delete data.traffic.length;
      
      } else if (data.traffic.toString() == "") {
        region.@tid = "null";
        region.@turl = "null";
        region.@tts = G_TIME_NOW;
      }
      
      if (!region.parent())
        regions.region += region;
    }
    
    //check label and etc.
    var id = this._observers[aButtonId].nodes[0].id;
    var props = this.getCityProps(id);
    
    var propsToUpdateObject = {};
    
    var labelFromXML = data.region.title.toString();
    if (labelFromXML && (labelFromXML != props.label))
      propsToUpdateObject.label = labelFromXML;
    
    if (aIsDataFromRequest &&
        props.trafficEnabledValue == "1" &&
        (data.traffic.length() == 0 || data.traffic.children().length() == 0)) {
      
      propsToUpdateObject.trafficEnabledValue = "";
    
    } else if (props.trafficEnabledValue == "" && (data.traffic.@region.toString() || data.traffic.nodata.length() == 1)) {
      propsToUpdateObject.trafficEnabledValue = "1";
    }
    
    this.updateButtonProps(id, propsToUpdateObject, true);
    
    this._observers[aButtonId].data = data;
    
    OBSERVER_SERVICE.notifyObservers(null, "Ya-City-Data-Changed", aButtonId);
  },
  
  _setOldData: function(aCityId, aButtonId, aSetIfNoData) {
    for each (var observer in this._observers) {
      if (observer.nodes[0]) {
        var props = this.getCityProps(observer.nodes[0].id);
        if (props.cityId === aCityId && observer.data) {
          this._setData(aCityId, observer.data);
          return true;
        }
      }
    }
    
    if (aSetIfNoData)
      this.__setData(aButtonId);
    
    return false;
  },
  
  addObserver: function(aCityNode) {
    var props = this.getCityProps(aCityNode.id);
    
    if (!this._observers[props.buttonId]) {
      this._observers[props.buttonId] = {nodes: [aCityNode]};
      
      this._setOldData(props.cityId, props.buttonId, true);
      
      if (!this.checkAndAddTimer(props.timerName, aCityNode.id) && gYaSearchService.isCountersAutoUpdateEnabled)
        gYaSearchService.refreshHTTPData("city", props);
      
    } else {
      this._observers[props.buttonId].nodes.push(aCityNode);
      this._setData(props.buttonId, this._observers[props.buttonId].data);
    }
  },
  
  removeObserver: function(aCityNode) {
    var props = this.getCityProps(aCityNode.id);
    
    if (props.buttonId && this._observers[props.buttonId]) {
      var cityNodes = this._observers[props.buttonId].nodes;
      
      for (var i = cityNodes.length; i-- > 0;)
        if (cityNodes[i] == aCityNode)
          cityNodes.splice(i,1);
      
      this.checkObserver(props);
    }
  },
  
  checkObserver: function(props) {
    var cityNodes = this._observers[props.buttonId].nodes;
    
    if (cityNodes.length == 0) {
      this.deleteTimer(props, 1);
      delete this._observers[props.buttonId];
    }
  },
  
  deleteTimer: function(aProps, aMinCount) {
    var count = 0;
    var city = aProps.cityId, timerName = aProps.timerName;
    
    for each (var observer in this._observers) {
      if (observer.nodes[0]) {
        var props = this.getCityProps(observer.nodes[0].id);
        if (props.cityId === city)
          count++;
      }
    }
    
    if (count < aMinCount) {
      gYaSearchService.clearTimer(timerName);
      
      delete gYaSearchService.checkTimeOut[timerName];
      delete gYaSearchService[timerName];
    }
  },
  
  checkAndAddTimer: function(aTimerName, aIdAttrValue) {
    var me = this;
    
    if (!gYaSearchService[aTimerName]) {
      gYaSearchService[aTimerName] = {
        notify: function(aTimer) {
          gYaSearchService.refreshHTTPData("city", me.getCityProps(aIdAttrValue));
        }
      };
      
      gYaSearchService.checkTimeOut[aTimerName] = gYaSearchService.isCountersAutoUpdateEnabled ? (gYaSearchService.getIntPref("yasearch.http.update.weathertraff.interval") * MIN_SEC || 0) : 0;
      
      return false;
    }
    
    return true;
  },
  
  updateObservers: function() {
    for each (var observer in this._observers) {
      var nodes = observer.nodes,
          check = false;
      
      for (var i = nodes.length; i-- > 0;) {
        if (!nodes[i].ownerDocument.getElementById(nodes[i].id)) {
          check = nodes[i].id;
          nodes.splice(i,1);
        }
      }
      
      if (check)
        this.checkObserver(this.getCityProps(check));
    }
  },
  
  updateObserversTime: function(aTime) {
    for each (var observer in this._observers) {
      var props = this.getCityProps(observer.nodes[0].id);
      gYaSearchService.checkTimeOut[props.timerName] = aTime;
      gYaSearchService.setTimer(props.timerName);
    }
  },
  
  updateCityInfo: function(aToolbarbuttonId, aForceRequest) {
    let props = this.getCityProps(aToolbarbuttonId);
    if (aForceRequest)
      props.forceRequest = true;
    
    gYaSearchService.manualRefreshHTTPData("city", props, "yasearch-city-" + props.buttonId);
  },
  
  updateAllCityInfo: function(aManual) {
    var fnName = aManual ? "manualRefreshHTTPData" : "refreshHTTPData";
    for each (var observer in this._observers)
      gYaSearchService[fnName]("city", this.getCityProps(observer.nodes[0].id));
  },
  
  updateButtonProps: function(aId, aNewProps, aDontForceRefresh) {
    if (!(typeof(aNewProps) == "object" && aNewProps.toSource && aNewProps.toSource() != "({})"))
      return false;
    
    var oldProps = this.getCityProps(aId);
    var newProps = this.getCityProps(aId);
    
    for (var prop in aNewProps)
      newProps[prop] = aNewProps[prop];
    
    if (newProps.cityId != oldProps.cityId) {
      newProps.labelAlias = "";
      
      if (!("weatherEnabledValue" in aNewProps))
        newProps.weatherEnabledValue = "";
      
      if (!("trafficEnabledValue" in aNewProps))
        newProps.trafficEnabledValue = "";
    }
    
    var newIdAttr = this._convertPropsObjectToString(newProps);
    newProps = this.getCityProps(newIdAttr);
    
    if (aId === newIdAttr)
      return;
    
    for each (var cityNode in this._observers[newProps.buttonId].nodes) {
      cityNode.setAttribute("id", newIdAttr);
      gYaSearchService._persistToolbarSet(cityNode.parentNode);
    }
    
    if (aDontForceRefresh)
      return;
    
    if (newProps.timerName != oldProps.timerName) {
      this._observers[oldProps.buttonId].data = null;
      this.deleteTimer(oldProps, 2);
      this.checkAndAddTimer(newProps.timerName, newIdAttr);
    }
    
    if (!this._setOldData(newProps.cityId, newProps.buttonId, false))
      gYaSearchService.setTimer(newProps.timerName, 1);
    
    OBSERVER_SERVICE.notifyObservers(null, "Ya-City-Data-Changed", newProps.buttonId);
  },
  
  _convertPropsObjectToString: function(aProps) {
    return "yasearch-city-" +
            [ aProps.buttonId,
              aProps.cityId,
              aProps.labelPure,
              aProps.weatherEnabledValue,
              aProps.trafficEnabledValue,
              aProps.labelAliasPure
            ].join("-");
  },
  
  getCityProps: function(aId) {
    var props = aId.split("-");
    
    function getPureLabelFromString(aString) {
      return aString.substr(0, 100)
                     .replace(/[^0-9A-Za-z\u0401-\u04e9\s\-\.,_]/gm, "")
                     .replace(/^\s+/, "")
                     .replace(/\s+$/, "")
                     .replace(/\s+/g, " ")
                     .replace(/-/g, '$1')
                     .replace(/,/g, '$2');
    }
    
    function unescapePureLabel(aString) {
      return aString.replace(/\$1/g, "-").replace(/\$2/g, ",");
    }
    
    return {
      buttonId: props[2],
      cityId: props[3] || "",
      
      labelPure:      props[4] || "",
      labelAliasPure: props[7] || "",
      
      get label()         { return unescapePureLabel(this.labelPure) },
      set label(val)      { this.labelPure = getPureLabelFromString(val) },
      
      get labelAlias()    { return unescapePureLabel(this.labelAliasPure) },
      set labelAlias(val) { this.labelAliasPure = getPureLabelFromString(val) },
      
      get buttonLabel() {
        return this.labelAlias || this.label;
      },
      
      get timerName() {
        return "_CityItemTimer" + this.cityId;
      },
      
      weatherEnabledValue: props[5] || "",
      trafficEnabledValue: props[6] || "",
      
      get weatherEnabled() {
        return (this.weatherEnabledValue != "0");
      },
      
      get trafficEnabled() {
        return (this.trafficEnabledValue == "1");
      }
    }
  },
  
  getDataForCityId: function(aId) {
    var data = this.getCityProps(aId);
    
    for each (var observer in this._observers) {
      if (observer.nodes[0]) {
        var props = this.getCityProps(observer.nodes[0].id);
        if (props.cityId === data.cityId && observer.data) {
          data.xmlData = observer.data;
          break;
        }
      }
    }
    
    if (!data.xmlData)
      data.xmlData = <info/>;
    
    return data;
  },
  
  __citiesDataFileName: "cities.data.xml",
  
  get dataFile() {
    var file = gYaSearchService.getYandexDir();
    file.append(this.__citiesDataFileName);
    
    if (file.exists() && file.isFile() && file.isReadable())
      return file;
    
    file = gYaSearchService.getContentDir();
    file.append("services");
    file.append(this.__citiesDataFileName);
    
    if (file.exists() && file.isFile() && file.isReadable())
      return file;
    
    return null;
  },
  
  get dataFileContent() {
    var dataFile = this.dataFile;
    if (!dataFile)
      return "";
    
    var dataContent = gYaSearchService.readFile(dataFile);
    
    if (dataContent)
      this.setTimeoutForClearGeoData();
    
    return dataContent;
  },
  
  __citiesData: null,
  get citiesData() {
    if (this.__citiesData === null) {
      var citiesData = this.dataFileContent;
      this.__citiesData = citiesData ? gYaSearchService.safeE4Xml(gYaSearchService.getDOMDocContent("cities-data", gYaSearchService.domParser.parseFromString(citiesData, "text/xml")), null, "regions") : null;
    }
    return this.__citiesData;
  },
  
  __timerForClearGeoData: null,
  
  clearTimeoutForClearGeoData: function() {
    if (this.__timerForClearGeoData)
      this.__timerForClearGeoData.cancel();
    
    this.__timerForClearGeoData = null;
  },
  
  setTimeoutForClearGeoData: function() {
    this.clearTimeoutForClearGeoData();
    
    var me = this;
    this.__timerForClearGeoData = new G_Timer(function() {
      me.clearGeoData();
    }, 15 * MIN_SEC)
  },
  
  clearGeoData: function() {
    this.clearTimeoutForClearGeoData();
    this.__citiesData = null;
    this.__weatherConverterXML = null;
  },
  
  get regionsDataFile() {
    var fileName = "regions.data.xml";
    var file = gYaSearchService.getYandexDir();
    file.append(fileName);
    
    if (!file.exists() || !file.isFile()) {
      try {
        file.create(Ci.nsIFile.NORMAL_FILE_TYPE, 0755);
      } catch(e) {}
    }
    
    if (file.exists() && file.isFile() && file.isReadable() && file.isWritable())
      return file;
    
    return null;
  },
  
  __regionsData: null,
  
  get regionsData() {
    if (this.__regionsData === null) {
      var file = this.regionsDataFile;
      var fileContent = file ? gYaSearchService.readFile(file) : "";
      
      var data = gYaSearchService.safeE4Xml(fileContent, "<data><regions/></data>", ["data"]);
      
      if (data.regions.length() != 1)
        data.regions = data.regions[0] || <regions/>;
      
      this.__regionsData = data;
    }
    
    return this.__regionsData;
  },
  
  flushRegionsData: function() {
    if (this.__regionsData === null)
      return false;
    
    var file = this.regionsDataFile;
    if (!file)
      return false;
    
    var regions = new XML("<regions/>");
    var timeMin = (G_TIME_NOW - DAY_SECS * 7).toString();
    
    for each (var region in this.__regionsData.regions.*)
      if (region.@tts.toString() > timeMin)
        regions.region += region;
    
    delete this.__regionsData.regions;
    this.__regionsData.regions[0] = regions;
    
    gYaSearchService.writeFile(file, this.__regionsData);
    this.__regionsData = null;
  },
  
  __weatherConverterXML: null,
  get weatherConverterFile() {
    var file = gYaSearchService.getContentDir();
    file.append("services");
    file.append("weather.convert.xml");
    return (file.exists() && file.isFile() && file.isReadable()) ? file : null;
  },
  
  get weatherConverterXML() {
    if (this.__weatherConverterXML === null) {
      var file = this.weatherConverterFile;
      if (file)
        this.__weatherConverterXML = gYaSearchService.safeE4Xml(gYaSearchService.readFile(file), null, "convert");
    }
    
    this.setTimeoutForClearGeoData();
    
    return this.__weatherConverterXML;
  },
  
  convertOldWeatherTBItemId: function(aOldWeatherTBItemId) {
    var defaultResult = "yasearch-city-" + (new Date().getTime()) + Math.floor(Math.random() * 100);
    
    var p = /^yasearch\-weather\-(\d+)(?:\-([^\-]*)(?:\-([^\-]*))?)?$/.exec(aOldWeatherTBItemId);
    if (!p || !p[2])
      return defaultResult;
    
    var aliasName = p[3] || "";
    if (aliasName === "##hidden##")
      aliasName = false;
    
    var city = p[2] || "";
    
    var regionId = "", regionName = "";
    
    var converterXML = this.weatherConverterXML;
    if (converterXML) {
      var region = converterXML..c.(@w == city)[0];
      if (region) {
        regionId = region.@g.toString();
        var citiesData = this.citiesData;
        if (citiesData) {
          region = citiesData..region.(@id == regionId)[0];
          if (region)
            regionName = region.@name.toString().replace(/-/g, '$1').replace(/,/g, '$2');
        }
      }
    }
    
    return [defaultResult, regionId, regionName, 1, "", aliasName].join("-");
  },
  
  __stringBundle: null,
  
  get stringBundle() {
    if (!this.__stringBundle) {
      var strBundleService = Cc["@mozilla.org/intl/stringbundle;1"].createInstance(Ci.nsIStringBundleService);
      this.__stringBundle = strBundleService.createBundle("chrome://yasearch/locale/city/city.properties");
    }
    return this.__stringBundle;
  },
  
  getString: function(aName) {
    return this.stringBundle.GetStringFromName(aName);
  },

  getFormattedString: function(aName, aStrArray) {
    return this.stringBundle.formatStringFromName(aName, aStrArray, aStrArray.length);
  }
}

gYaCity._init();