var gYaOverlay = {
  _yaToolbarId: "yasearch-bar",
  _yaIdPrefix: "yasearch",
  _yaIdPrefixRE: /^yasearch\-/,
  
  _inited: false,
  
  init: function Overlay_init() {
    if (!this._inited) {
      this._inited = true;
      Components.utils.import("resource://yasearch/bar.protocol.jsm", this);
      this.ProtocolHandler.addHandler(this.barProtocolHandler);
    }
  },
  
  shutdown: function Overlay_destroy() {
    if (this._inited) {
      this._inited = false;
      this.ProtocolHandler.removeHandler(this.barProtocolHandler);
    }
  },
  
  get _rdf() {
    delete this._rdf;
    return this._rdf = Cc["@mozilla.org/rdf/rdf-service;1"].getService(Ci.nsIRDFService);
  },
  
  get _dataSource() {
    delete this._dataSource;
    return this._dataSource = this._rdf.GetDataSource("rdf:local-store");
  },
  
  getRDFLiteralValue: function Overlay_getRDFLiteralValue(aSource, aProperty) {
    let target = this._dataSource.GetTarget(aSource, aProperty, true);
    if (target instanceof Ci.nsIRDFLiteral)
      return target.Value;

    return null;
  },
  
  setRDFLiteralValue: function Overlay_setRDFLiteralValue(aSource, aProperty, aTarget) {
    try {
      var oldTarget = this._dataSource.GetTarget(aSource, aProperty, true);
      if (oldTarget) {
        if (aTarget)
          this._dataSource.Change(aSource, aProperty, oldTarget, this._rdf.GetLiteral(aTarget));
        else
          this._dataSource.Unassert(aSource, aProperty, oldTarget);
      } else {
        this._dataSource.Assert(aSource, aProperty, this._rdf.GetLiteral(aTarget), true);
      }
    } catch(e) {}
  },
  
  get defaultset() {
    delete this.defaultset;
    return this.defaultset = this.overlayDocument.getElementById(this._yaToolbarId).getAttribute("defaultset");
  },
  
  setToolbarCollapsedState: function Overlay_setCollapsedState(aCollapsedState) {
    if (typeof aCollapsedState !== "boolean")
      return;
    
    let newCollapsedState = aCollapsedState.toString();
    
    try {
      let collapsedResource = this._rdf.GetResource("collapsed");
      let toolbarResource = this._rdf.GetResource("chrome://browser/content/browser.xul#yasearch-bar");
      let currentCollapsedState = this.getRDFLiteralValue(toolbarResource, collapsedResource);
      
      if ((currentCollapsedState || "false").toString() != newCollapsedState) {
        this.setRDFLiteralValue(toolbarResource, collapsedResource, newCollapsedState);
        this._dataSource.QueryInterface(Ci.nsIRDFRemoteDataSource).Flush();
      }
    } catch(e) {}
  },
  
  get _CurrentsetIds() {
    delete this._CurrentsetIds;
    
    this._CurrentsetIds = new G_HashTable();
    
    let allResources = this._dataSource.GetAllResources();
    let currentsetResource = this._rdf.GetResource("currentset");
    
    let re = this._yaIdPrefixRE;
    
    while (allResources.hasMoreElements()) {
      let res = allResources.getNext().QueryInterface(Ci.nsIRDFResource);
      let tool = res.Value;
      
      if (tool) {
        let toolbar = this._rdf.GetResource(tool);
        let currentSet = this.getRDFLiteralValue(toolbar, currentsetResource);
        if (currentSet) {
          for each (let elemId in currentSet.split(",")) {
            if (re.test(elemId)) {
              this._CurrentsetIds.put(elemId, true);
            }
          }
        }
      }
    }
    
    for each (let elemId in this.defaultset.split(",")) {
      if (re.test(elemId)) {
        this._CurrentsetIds.put(elemId, true);
      }
    }
    
    return this._CurrentsetIds;
  },
  
  get CurrentsetIds() {
    this._refreshCurrentsetIds();
    return this._CurrentsetIds;
  },
  
  _refreshCurrentsetIds: function() {
    let browser = null,
        topBrowser = gYaSearchService.getWindow("navigator:browser"),
        enumerator = gYaSearchService.windowMediator.getEnumerator("navigator:browser");
    
    while (browser == null && enumerator.hasMoreElements()) {
      let br = enumerator.getNext();
      if (br !== topBrowser)
        browser = br;
    }
    
    if (browser && browser.document) {
      let nodes = gYaSearchService.DOMUtils.evaluateXPath(browser.document,
                  '//*[@yadouble][starts-with(@id,"yasearch-")]');
      
      for each (let node in nodes) {
        this._CurrentsetIds.put(node.getAttribute("id"), true);
      }
    }
  },
  
  get ClonesData() {
    delete this.ClonesData;
    
    this.ClonesData = new G_HashTable();
    
    let re = this._yaIdPrefixRE;
    
    this.ClonesData.getNodeForId = function(aId) {
      if (!(typeof aId == "string" && re.test(aId)))
        return null;
      
      return this.get(aId.split(/\-\d/)[0]);// "yasearch-smthng-1234.." => "yasearch-smthng"
    }
    
    this.ClonesData.getCloneForId = function(aId) {
      let cloned = null;
      let node = this.getNodeForId(aId);
      
      if (node) {
        cloned = node.cloneNode(true);
        cloned.setAttribute("yadouble", "child");
        cloned.setAttribute("id", aId);
      }
      
      return cloned;
    }
    
    let elements =
        Array.slice(this.overlayDocument.getElementsByTagName("toolbaritem"))
             .filter(function(el) {
               return el.getAttribute("yadouble") == "parent" && re.test(el.getAttribute("id"));
              });
    
    for each (var el in elements) {
      this.ClonesData.put(el.getAttribute("id"), el);
    }
    
    return this.ClonesData;
  },
  
  get overlayDocument() {
    delete this.overlayDocument;
    
    const XULNS = "http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul";
    
    let overlayFile = gYaSearchService.getContentDir();
    overlayFile.append("yasearch.xul");
    
    let overlayData = gYaSearchService.readFile(overlayFile);
    let overlayDoc = gYaSearchService.domParser.parseFromString(overlayData, "text/xml");
    
    return this.overlayDocument = overlayDoc;
  },
  
  insertToolbarItem: function(aId, aBeforeElement, aBrowser) {
    if ((aBeforeElement instanceof Ci.nsIDOMElement) && !aBrowser)
      return [];
    
    let newItemId = aId + "-" + (new Date().getTime());
    let clone = this.ClonesData.getCloneForId(newItemId);
    
    if (!clone)
      return [];
    
    function getBeforeElement(aToolbar) {
      let node;
      
      if (aBeforeElement) {
        if (aBeforeElement instanceof Ci.nsIDOMElement)
          node = aBeforeElement;
        else
          node = aToolbar.getElementsByAttribute("id", aBeforeElement)[0];
      }
      
      return node;
    }
    
    let browsers = typeof aBrowser != "undefined" ? [aBrowser] : gYaSearchService.getWindows("navigator:browser");
    
    let newItemNode = null;
    
    for each (let browser in browsers) {
      let toolbar = browser.document.getElementById(this._yaToolbarId);
      if (toolbar) {
        newItemNode = toolbar.ownerDocument.importNode(clone.cloneNode(true), true);
        let beforeElt = getBeforeElement(toolbar);
        
        if (beforeElt)
          toolbar.insertBefore(newItemNode, beforeElt);
        else
          toolbar.appendChild(newItemNode);
      }
    }
    
    //2do: also need remove
    //this._CurrentsetIds.put(newItemId, true);
    
    return [newItemId, newItemNode];
  },
  
  barProtocolHandler: {
    canHandleSpec: function(aSpec) {
      return aSpec === "browser-overlay";
    },

    newURI: function(aSpec, aOriginalCharset, aBaseURI) {
      if (aSpec === "browser-overlay") {
        try {
          let content = btoa(this._docontent());
          
          let uri =  Cc["@mozilla.org/network/simple-uri;1"].createInstance(Ci.nsIURI);
          uri.spec = "data:application/vnd.mozilla.xul+xml;base64," + content;
          
          return uri;
        } catch(e) {
          gYaSearchService.log("gYaOverlay.newURI: can't create content URI\n" + e);
        }
      }

      return null;
    },

    newChannel: function(aURI) {
      return null;
    },
    
    _docontent: function() {
      let clonedOverlayDoc = gYaSearchService.domParser.parseFromString(' \
          <overlay xmlns="http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul"> \
            <toolbarpalette id="BrowserToolbarPalette"/> \
          </overlay>', "text/xml");
      
      let toolbarPalette = clonedOverlayDoc.getElementById("BrowserToolbarPalette");
      
      let clonesData = gYaOverlay.ClonesData;
      
      //gYaSearchService.log("CurrentsetIds: " + gYaOverlay.CurrentsetIds.keys)
      
      for (let [id,] in gYaOverlay.CurrentsetIds) {
        let clone = clonesData.getCloneForId(id);
        if (clone)
          toolbarPalette.appendChild(clone);
      }
      
      return UConverter.ConvertFromUnicode(gYaSearchService.xmlSerializer.serializeToString(clonedOverlayDoc));
    }
  }
};