const Cc = Components.classes;
const Ci = Components.interfaces;

function nsIYaCityAComplete() {
  this.wrappedJSObject = this;
  
  this.citiesData = null;
  
  this._lastResult = null;
  this.clearLastResult();
};

nsIYaCityAComplete.prototype = {
  get nsIYa() {
    if (!this._nsIYa)
      this._nsIYa = Cc["@yandex.ru/yasearch;1"].getService(Ci.nsIYaSearch).wrappedJSObject;
    
    return this._nsIYa;
  },
  
  get lastResult() {
    return this._lastResult;
  },
  
  set lastResult(val) {
    this._lastResult = val ? val : {id: 0, city: null};
  },
  
  clearLastResult: function() {
    this.lastResult = null;
  },
  
  _clearCitiesListTimer: null,
  
  timedClearCitiesList: function() {
    if (this._clearCitiesListTimer)
      this._clearCitiesListTimer.cancel();
    else
      this._clearCitiesListTimer = Cc["@mozilla.org/timer;1"].createInstance(Ci.nsITimer);
    
    let me = this;
    this._clearCitiesListTimer.initWithCallback(
      {
        notify: function() {
          me.citiesData = null;
        }
      }, 5 * 60 * 1000, Ci.nsITimer.TYPE_ONE_SHOT
    );
  },
  
  compareCities: function(a, b) {
    return (a.city.toLowerCase() < b.city.toLowerCase()) ? -1 : (a.city > b.city) ? 1 : 0;
  },
  
  get citiesList() {
    if (!this.citiesData) {
      var cList = this.nsIYa.yaCity.citiesData;
      if (!cList || !cList.regions)
        return false;
      
      this.citiesData = [];
      
      for each (let region in cList..region) {
        this.citiesData.push({
          city: region.@name.toString(),
          cityid: region.@id.toString(),
          country: region.parent().@name.toString()
        });
      }
      
      this.citiesData.sort(this.compareCities);
    }
    
    return this.citiesData;
  },
  
  startSearch: function(searchString, searchParam, previousResult, listener) {
    this.clearLastResult();
    
    if (!this.citiesList)
      return;
    
    let candidats = [],
        re = new RegExp("^" + searchString.replace(/(\.|\||\+|\?|\$|\(|\)|\[|\])/g,"\\\$1"), "i");
    
    for each (let data in this.citiesList)
      if (re.test(data.city))
        candidats.push(data);
    
    this.timedClearCitiesList();
    
    let result = {
      QueryInterface: function(aIID) {
        if (aIID.equals(Ci.nsIAutoCompleteSearch) || aIID.equals(Ci.nsISupports))
          return this;
        throw Components.results.NS_NOINTERFACE;
      },
      
      results: candidats,
      defaultIndex: 0,
      errorDescription: null,
      matchCount: candidats.length,
      searchResult: Ci.nsIAutoCompleteResult[candidats.length ? "RESULT_SUCCESS" : "RESULT_NOMATCH"],
      searchString: searchString,
      
      getCommentAt: function(index) {
        return this.results[index].country;
      },
      
      getStyleAt: function(index) {
        return null;
      },
      
      getImageAt: function(index) {
        return "";
      },
      
      getValueAt: function(index) {
        let res = this.results[index];
        gYaCityAutoComplete.lastResult = {id: res.cityid, city: res.city};
        return res.city;
      },
      
      removeValueAt: function(rowIndex, removeFromDb) {}
    }
    
    this.lastResult = candidats;
    
    listener.onSearchResult(this, result);
  },
  
  stopSearch: function(x) {},
  
  prepeareForNewSearch: function() {
    this.clearLastResult();
    this.citiesList;
  },
  
  QueryInterface: function(aIID) {
    if (!aIID.equals(Ci.nsIAutoCompleteSearch) &&
        !aIID.equals(Ci.nsISupports))
      throw Components.results.NS_ERROR_NO_INTERFACE;
    return this;
   },
  
  module: {
    _firstTime: true,
    
    registerSelf: function (compMgr, fileSpec, location, type) {
      
      if (this._firstTime) {
        this._firstTime = false;
        throw Components.results.NS_ERROR_FACTORY_REGISTER_AGAIN;
      };
      
      var compReg = compMgr.QueryInterface( Ci.nsIComponentRegistrar );
      compReg.registerFactoryLocation(this.cid,
                                      "YasearchCity Autocomplete",
                                      this.contractId,
                                      fileSpec, location, type);
    },
    
    unregisterSelf: function(compMgr, fileSpec, location) {
      compMgr = compMgr.QueryInterface( Ci.nsIComponentRegistrar );
      compMgr.unregisterFactoryLocation( this.cid, fileSpec );
    },
    
    getClassObject: function (compMgr, cid, iid) {
      if (!cid.equals(this.cid))
        throw Components.results.NS_ERROR_NO_INTERFACE;
      if (!iid.equals(Ci.nsIFactory))
        throw Components.results.NS_ERROR_NOT_IMPLEMENTED;
      return this.factory;
    },
    
    cid: Components.ID("{DE203E54-C244-4c17-AB02-F69B6ECFA59D}"),
    
    contractId: "@mozilla.org/autocomplete/search;1?name=YasearchCity",
    
    factory: {
      createInstance: function (outer, iid) {
        if (outer != null)
          throw Components.results.NS_ERROR_NO_AGGREGATION;
        return gYaCityAutoComplete;
      }
    },
  
    canUnload: function(compMgr) {return true;}
  }
};

function NSGetModule(compMgr, fileSpec) {
  return nsIYaCityAComplete.prototype.module;
}

var gYaCityAutoComplete = new nsIYaCityAComplete();