const Cc = Components.classes;
const Ci = Components.interfaces;

function nsIYaPassManager() {
  this.wrappedJSObject = this;
};

nsIYaPassManager.prototype = {
  _CID: Components.ID("{7a19ec9e-54e4-4099-ae92-ac3628c67889}"),
  _contractID: "@mozilla.org/autocomplete/search;1?name=YasearchPassComplete",
  _className: "YasearchPassManager Autocomplete",

  QueryInterface: function(aIID) { 
    if (!aIID.equals(Ci.nsIAutoCompleteSearch) &&
        !aIID.equals(Ci.nsISupports))
      throw Components.results.NS_ERROR_NO_INTERFACE;
    return this;
  },
  
  get loginManager() {
    return Cc["@mozilla.org/login-manager;1"].getService(Ci.nsILoginManager);
  },
  
  _defaultUserPrefName: "yasearch.users.default",
  
  get onHoverTreeUglyHack() {
    return this.__onHoverTreeUglyHack;
  },
  
  set onHoverTreeUglyHack(val) {
    this.__onHoverTreeUglyHack = !!val;
    return this.__onHoverTreeUglyHack;
  },
  
  startSearch: function(aSearchString, aSearchParam, aPreviousResult, aListener) {
    var candidats = [];
    
    if (aSearchString) {
      for each (var login in this.savedLogins)
        if (login.indexOf(aSearchString) == 0)
          candidats.push(login);
    } else {
      candidats = this.savedLogins;
    }
    
    var _me = this;
    
    var result = {
      QueryInterface: function(aIID) {
        if (aIID.equals(Ci.nsIAutoCompleteSearch) || aIID.equals(Ci.nsISupports))
          return this;
        
        throw Components.results.NS_NOINTERFACE;
      },
      
      results: candidats,
      defaultIndex: 0,
      errorDescription: null,
      matchCount:    candidats.length,
      searchResult:  Ci.nsIAutoCompleteResult[candidats.length ? "RESULT_SUCCESS" : "RESULT_NOMATCH"],
      searchString:  aSearchString,
      getCommentAt:  function(index) { return ""; },
      getStyleAt:    function(index) {
        return _me.onHoverTreeUglyHack ? "commentColumnHover" : "";
      },
      getImageAt: function(index) { return ""; },
      getValueAt:    function(index) { return this.results[index]; },
      removeValueAt: function(rowIndex, removeFromDb) {}
    }
    
    aListener.onSearchResult(this, result);
  },
  
  stopSearch: function() {},
  
  _loginChromeUrl: "chrome://yasearch/",
  
  _savedLogins: null,
  
  resetSavedLogins: function() {
    this.savedLogins = null;
  },
  
  set savedLogins(val) {
    if (val !== null)
      throw "YasearchPassComplete.savedLogins must be null";
    
    this._savedLogins = null;
  },
  
  get savedLogins() {
    if (!this._savedLogins) {
      let logins = [];
      
      let storedLogins = this.loginManager.findLogins({}, this._loginChromeUrl, this._loginChromeUrl, null);
      
      for (let i = 0, len = storedLogins.length; i < len; i++) {
        let l = storedLogins[i];
        logins.push(l.username);
      }
      
      this._savedLogins = logins.sort();
    }
    
    return this._savedLogins;
  },
  
  get hasSavedLogins() {
    return !!this.savedLogins.length;
  },
  
  // case sensitive; "'.' vs '-'"; "login@smthng"
  _prepeareUsername: function(aUsername) {
    function toLowerWithoutDots(aString) {
      return aString.toLowerCase().replace(/\./g, "-");
    }
    
    let usernameLower = toLowerWithoutDots(aUsername);
    let savedLogins = this.savedLogins;
    
    return savedLogins.filter(function(aLogin) {
             return aLogin === aUsername;
           })[0] ||
           savedLogins.filter(function(aLogin) {
             return toLowerWithoutDots(aLogin) === usernameLower;
           })[0] ||
           savedLogins.filter(function(aLogin) {
             return aLogin.split("@")[0] === aUsername;
           })[0] ||
           savedLogins.filter(function(aLogin) {
             return toLowerWithoutDots(aLogin).split("@")[0] === usernameLower;
           })[0] ||
           aUsername;
  },
  
  _loginManagerFindLogin: function(aUsername) {
    let logins = this.loginManager.findLogins({}, this._loginChromeUrl, this._loginChromeUrl, null);
    
    for (var i = 0, len = logins.length; i < len; i++) {
      let username = logins[i].username;
      if (username === aUsername || username.split("@")[0] === aUsername) {
        return logins[i];
      }
    }
    
    return null;
  },
  
  _removeUserData: function(aUsername) {
    let login = this._loginManagerFindLogin(aUsername);
    
    if (login)
      this.loginManager.removeLogin(login);
  },
  
  removeUserData: function(aUsername) {
    let username = this._prepeareUsername(aUsername);
    
    this.resetSavedLogins();
    
    this._removeUserData(username);
    
    try {
      const prefBranch = Cc["@mozilla.org/preferences-service;1"].getService(Ci.nsIPrefBranch);
      if (username == prefBranch.getCharPref(this._defaultUserPrefName))
        prefBranch.clearUserPref(this._defaultUserPrefName);
    } catch(e) {}
  },
  
  getStoredPassword: function(aUsername) {
    let login = this._loginManagerFindLogin(this._prepeareUsername(aUsername));
    return login ? login.password : null;
  },
  
  removeAllUsersData: function() {
    this.savedLogins.forEach(function(aUsername) {
      this._removeUserData(aUsername);
    }, this);
    
    try {
      const prefBranch = Cc["@mozilla.org/preferences-service;1"].getService(Ci.nsIPrefBranch);
      prefBranch.clearUserPref(this._defaultUserPrefName);
    } catch(e) {}
    
    this.resetSavedLogins();
  },
  
  storeLoginDetails: function(aUsername, aPassword, aStore) {
    this.removeUserData(aUsername);
    
    if (aStore) {
      const nsLoginInfo = new Components.Constructor("@mozilla.org/login-manager/loginInfo;1", Ci.nsILoginInfo, "init");
      var loginInfo = new nsLoginInfo(this._loginChromeUrl, this._loginChromeUrl, null, aUsername, aPassword, "", "");
      
      this.loginManager.addLogin(loginInfo);
      
      const prefBranch = Cc["@mozilla.org/preferences-service;1"].getService(Ci.nsIPrefBranch);
      prefBranch.setCharPref(this._defaultUserPrefName, aUsername);
    }
  }
}; 

var objects = [nsIYaPassManager]; 

function FHolder(aObj) { 
  this.CID        = aObj.prototype._CID; 
  this.contractID = aObj.prototype._contractID; 
  this.className  = aObj.prototype._className; 
  
  this.factory = {
    createInstance: function(aOuter, aIID) {
      if (aOuter !== null)
        throw Components.results.NS_ERROR_NO_AGGREGATION;
      return (new this.constructor).QueryInterface(aIID);
    }
  };
  
  this.factory.constructor = aObj; 
} 

var gModule = {
  _objects: {},
  
  registerSelf: function (aComponentManager, aFileSpec, aLocation, aType) {
    aComponentManager.QueryInterface(Ci.nsIComponentRegistrar);
    for (var key in this._objects) {
      var obj = this._objects[key];
      aComponentManager.registerFactoryLocation(obj.CID, obj.className, obj.contractID, aFileSpec, aLocation, aType); 
    }
  },
  
  unregisterSelf: function(aComponentManager, aFileSpec, aLocation) {
    aComponentManager.QueryInterface(Ci.nsIComponentRegistrar);
    for (var key in this._objects) {
      var obj = this._objects[key];
      aComponentManager.unregisterFactoryLocation(obj.CID, aFileSpec);
    }
  },
  
  getClassObject: function(aComponentManager, aCID, aIID) {
    if (!aIID.equals(Ci.nsIFactory))
      throw Components.results.NS_ERROR_NOT_IMPLEMENTED;
    
    for (var key in this._objects) {
      if (aCID.equals(this._objects[key].CID))
        return this._objects[key].factory;
    }
    
    throw Components.results.NS_ERROR_NO_INTERFACE;
  },
  
  canUnload: function(aComponentManager) {
    return true;
  }
};

function NSGetModule(compMgr, fileSpec) {
  for (var i in objects)
    gModule._objects[i] = new FHolder(objects[i]);
  
  return gModule;
}